alphabet = {
  "vowels": {
      "a": "අ",
      "A": "අ",
      "aa": "ආ",
      "Aa": "ආ",
      "ae": "ඇ",
      "Ae": "ඇ",
      "aee": "ඈ",
      "Aee": "ඈ",
      "i": "ඉ",
      "I": "ඉ",
      "ii": "ඊ",
      "Ii": "ඊ",
      "u": "උ",
      "U": "උ",
      "uu": "ඌ",
      "Uu": "ඌ",
      "ERU": "ඍ",
      "ERU'": "ඎ",
      "IRU": "ඏ",
      "IRU'": "ඐ",
      "e": "එ",
      "E": "එ",
      "ee": "ඒ",
      "Ee": "ඒ",
      "ai": "ඓ",
      "Ai": "ඓ",
      "o": "ඔ",
      "O": "ඔ",
      "oo": "ඕ",
      "Oo": "ඕ",
      "au": "ඖ",
      "Au": "ඖ",
      "x": "(අං)",
      "X": "(අඃ)"
  },
  "consonants": {
      "k": "ක",
      "c": "ක",
      "K": "ක",
      "kh": "ඛ",
      "Kh": "ඛ",
      "g": "ග",
      "G": "ඝ",
      "gh": "ඝ",
      "Gh": "ඝ",
      "ng": "ඞ",
      "nng": "ඟ",
      "zg": "ඟ",
      "ch": "ච",
      "Ch": "ඡ",
      "j": "ජ",
      "J": "ඣ",
      "jh": "ඣ",
      "ngj": "ඤ",
      "zk": "ඤ",
      "ny": "ඤ",
      "Ngj": "ඥ",
      "jny": "ඥ",
      "nyj": "ඦ",
      "zj": "ඦ",
      "t": "ට",
      "T": "ඨ",
      "d": "ඩ",
      "D": "ඪ",
      "N": "ණ",
      "zd": "ඬ",
      "nd": "ඬ",
      "th": "ත",
      "Th": "ථ",
      "dh": "ද",
      "Dh": "ධ",
      "n": "න",
      "ndh": "ඳ",
      "p": "ප",
      "P": "ඵ",
      "ph": "ඵ",
      "r": "ර",
      "l": "ල",
      "L": "ළ",
      "s": "ස",
      "sh": "ශ",
      "Sh": "ෂ",
      "h": "හ",
      "w": "ව",
      "v": "ව",
      "f": "ෆ",
      "b": "බ",
      "B": "භ",
      "bh": "භ",
      "m": "ම",
      "mb": "ඹ",
      "y": "ය"
  },
  "dependent_vowels": {
      "aa": "ා",
      "ae": "ැ",
      "aae": "ෑ",
      "i": "ි",
      "ii": "ී",
      "u": "ු",
      "uu": "ූ",
      "ERU": "ෘ",
      "e": "ෙ",
      "ee": "ේ",
      "ai": "ෛ",
      "o": "ො",
      "oo": "ෝ",
      "au": "ෞ",
      "x": "ං"
  }
}

config_data = {
  "api_key": "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",
  "org_key": "org-FAg23PQBtCvq57kZHYd0HYlW",
  "model": "gpt-3.5-turbo",
  "temperature": 0,
  "max_tokens": 2000,
  "Top_P": 1,
  "Frequency_penalty": 0,
  "Presence_penalty": 0,
  "max_characters": 4000,
  "TC_Only": "NO",
  "Prompts": [
    {
      "role": "user",
      "content": "I can provide you a sentence with some spelling errors. Your goal is to accurately identify the misspelled Sinhala words. \n\nThe output should be presented in JSON format, structured as follows:\n\n{\n    \"word_list\": [\n          \"<first_incorrect_word>\",\n          \"<second_incorrect_word>\",\n          ...............,\n          \"<last_incorrect_word>\"\n     ]\n}\nMake sure to use only the words which is not exist in the Sinhala language as \"<first_incorrect_word>\", \"<second_incorrect_word>\",  ..............., \"<last_incorrect_word>\".\n\nSentence: '{{masked-sentence}}'"
    },
    {
      "role": "user",
      "content": "I can provide you with a version of the word with some spelling errors. Your goal is to accurately identify the most suitable Sinhala word that matches the given misspelled version for the specified <mask>. Please note that the word you suggest should indeed exist in the Sinhala language (Real Word). Decide the category of the misspelled word as well. Word categories are subject, predicate, object, complement, and modifier. Get the help of this to decide the word as well.\n\nThe output should be presented in JSON format, structured as follows:\n\n{\n    \"<misspelled_word>\": \"<correct_word>\",\n     \"category\": \"<category_from_given_list>\"\n}\n\nSentence with the <mask>: '{{masked-sentence}}'\nMisspelled word: '{{misspelled-word}}'"
    }
  ]
}




import speech_recognition as sr
# Initialize recognizer class (for recognizing the speech)
r = sr.Recognizer()


def conversion(path: str):
    """
    Convert a speech audio file to text.

    This function reads an audio file and performs speech recognition using the Google Speech Recognition API.
    The recognized text is printed to the console.

    Parameters:
    -----------
    path : str
        The path to the audio file. Default is 'r../../resources/datasets/IT20167264/test_audio/pn_sin_01_00003.wav'.

    Returns:
    --------
    None

    Raises:
    -------
    None

    Notes:
    ------
    - Make sure you have the required dependencies installed, such as the SpeechRecognition library.
    - The language parameter can be modified to specify a different language for speech recognition.
    - The audio file should be in a compatible format supported by the SpeechRecognition library.
    """
    lang = 'si-LK'

    with sr.AudioFile(path) as source:
        print('Fetching File')
        audio_text = r.listen(source)
        try:
            print('Converting audio transcripts into text ...')
            text = r.recognize_google(audio_text, language=lang)
            return text
        except Exception as e:
            print(f'Sorry.. run again...{str(e)}')


def conversion_by_input():
    """
    Perform speech-to-text grammar_rule on an audio file.

    This function uses the SpeechRecognition library to recognize the speech in an audio file and convert it into text.

    Parameters:
    -----------
    None

    Returns:
    --------
    None

    Raises:
    -------
    UnknownValueError:
        If the speech recognition system could not understand the audio.
    RequestError:
        If there was an error during the speech recognition request.


    Notes:
    ------
    - Make sure you have the required dependencies installed, such as the SpeechRecognition library and the required speech recognition API keys.
    - The audio file should be in a compatible format supported by the SpeechRecognition library.
    - The language parameter can be modified to specify a different language for speech recognition.

    """
    # Initialize recognizer class (for recognizing the speech)
    recognizer = sr.Recognizer()

    # Read the audio file path from user input
    audio_file = input("Enter the path to the audio file: ")

    # Perform speech recognition
    with sr.AudioFile(audio_file) as source:
        print('Fetching File')
        audio_text = recognizer.listen(source)
        try:
            print('Converting audio transcripts into text ...')
            text = recognizer.recognize_google(audio_text, language='si-LK')
            return text
        except sr.UnknownValueError:
            print('Speech recognition could not understand audio')
        except sr.RequestError as e:
            print(f'Error occurred during speech recognition: {e}')



import json
import os
import pkg_resources
from sinlingua.src.singlish_resources import alphabet


class RuleBasedTransliterator:
    def __init__(self):
        """
        Initialize the RuleBasedTransliterator object.

        This constructor initializes the RuleBasedTransliterator object with the configuration data
        for vowels, consonants, and dependent vowels. It sets up the dictionaries needed for the
        transliteration process.

        Examples:
        ---------
        >>> from sinlingua.singlish.rulebased_transliterator import RuleBasedTransliterator
        >>>
        >>> transliterator = RuleBasedTransliterator()
        >>>
        >>> # The object is now initialized and ready to perform transliteration.
        """
        self.vowels = {}
        self.consonants = {}
        self.dependent_vowels = {}

        # alphabet = "singlish-alphabet.json"
        # data = self.__read_json_config(file_path=alphabet)
        data = alphabet
        self.vowels = data.get('vowels', {})
        self.consonants = data.get('consonants', {})
        self.dependent_vowels = data.get('dependent_vowels', {})

    def __split_into_logical_groups(self, word: str) -> list:
        logical_groups = []
        current_group = ''
        i = 0
        while i < len(word):
            try:
                for length in range(3, 0, -1):
                    if i + length <= len(word):
                        group = word[i:i + length]
                        if group in self.vowels:
                            logical_groups.append(self.vowels[group])
                            i += length

                for length in range(4, 0, -1):
                    if i + length <= len(word):
                        group = word[i:i + length]
                        if group in self.consonants:
                            logical_groups.append(current_group)
                            current_group = self.consonants[group]
                            i += length

                            if i >= len(word):
                                current_group = current_group + "්"
                                break
                            else:
                                for next_length in range(3, 0, -1):
                                    if i + next_length <= len(word):
                                        group = word[i:i + next_length]
                                        if group in self.dependent_vowels:
                                            logical_groups.append(current_group)
                                            current_group = self.dependent_vowels[group]
                                            i += next_length
                                            break
                                else:
                                    if i < len(word) and word[i] == "a":
                                        i += 1
                                    elif (word[i:i + 1] or word[i:i + 2] or word[i:i + 3] or word[i:i + 4]) in (self.consonants or self.vowels):
                                        current_group = current_group + "්"
                                        break
                                    else:
                                        # Handle unknown characters
                                        current_group += word[i]
                                        i += 1
                            break
                else:
                    # Handle unknown characters
                    current_group += word[i]
                    i += 1

                # Check for vowels
                for length in range(3, 0, -1):
                    if i + length < len(word):
                        group = word[i:i + length]
                        if group in self.vowels:
                            logical_groups.append(current_group)
                            current_group = self.vowels[word[i:i + 3]]
                            i += 3
                            break

                # # Check for dependent vowel signs
                # for length in range(3, 0, -1):
                #     print("Dependent Vowels")
                #     if i + length < len(word):
                #         group = word[i:i + length]
                #         if group in self.dependent_vowels:
                #             logical_groups.append(current_group)
                #             current_group = self.dependent_vowels[word[i:i + 3]]
                #             i += 3
                #             break

            except Exception as e:
                # Handle exceptions
                print(f"An error occurred: {str(e)}")
                return []

        if current_group:
            logical_groups.append(current_group)

        return logical_groups

    # @staticmethod
    # def __read_json_config(file_path: str) -> dict:
    #     try:
    #         json_file_path = pkg_resources.resource_filename('sinlingua', os.path.join('resources', file_path))
    #         # Read JSON configuration file and return the data as dictionary
    #         with open(os.path.join(json_file_path, file_path), 'r', encoding='utf-8') as json_file:
    #             json_data_c = json.load(json_file)
    #         return json_data_c
    #     except Exception as e:
    #         # Handle exceptions while reading JSON configuration
    #         print(f"Error while reading JSON configuration file '{file_path}': {str(e)}")
    #         return {}

    def transliterator(self, text: str) -> str:
        """
        Transliterate English text into the Sinhala script.

        This method takes English text as input and transliterates it into the Sinhala script.
        It separates the text into paragraphs and words, and transliterates each word individually.
        The transliterated text maintains the original paragraph structure.

        Parameters:
        -----------
        text : str
            The English text to be transliterated.

        Returns:
        --------
        str
            The transliterated text in the Sinhala script.

        Examples:
        ---------
        >>> from sinlingua.singlish.rulebased_transliterator import RuleBasedTransliterator
        >>>
        >>> transliterator = RuleBasedTransliterator()
        >>> input_phrase = "oyaata kohomadha"
        >>>
        >>> out = transliterator.transliterator(text=input_phrase)
        >>> print(out)
        ඔයාට කොහොමද

        Notes:
        ------
        - The method relies on the 'vowels', 'consonants', and 'dependent_vowels' dictionaries
          to perform the transliteration. Make sure these dictionaries are defined correctly
          in the configuration file before calling this method.
        - The method uses the 'split_into_logical_groups' helper function to split each word
          into logical groups based on the rules of Sinhala transliteration.
        """

        try:
            paragraphs = text.split('\n')
            transliterated_paragraphs = []

            for paragraph in paragraphs:
                words = paragraph.split()
                transliterated_words = []

                for word in words:
                    logical_groups = self.__split_into_logical_groups(word)

                    output = ''
                    for group in logical_groups:
                        if group in self.vowels:
                            output += self.vowels[group]
                        elif group in self.dependent_vowels:
                            output += self.dependent_vowels[group]
                        else:
                            output += group

                    transliterated_word = output
                    transliterated_words.append(transliterated_word)

                transliterated_paragraph = ' '.join(transliterated_words)
                transliterated_paragraphs.append(transliterated_paragraph)

            output_text = '\n'.join(transliterated_paragraphs)
            return output_text

        except Exception as e:
            print(f"Error in transliterator method: {e}")
            return text


import re
from typing import Tuple, Text, Dict, List


Boolean = bool

__all__ = [
    'SinhalaTokenizer',
]


def is_a_sinhala_letter(s: Text) -> Boolean:
    if len(s) != 1:
        return True
    sinhala_lower_bound = 3456
    sinhala_upper_bound = 3583
    cp = ord(s[0])  # first letter of str
    if sinhala_lower_bound <= cp <= sinhala_upper_bound:
        return True
    return False


def contains_sinhala(s: Text) -> Boolean:
    for c in s:
        if is_a_sinhala_letter(c):
            return True
    return False


# noinspection SpellCheckingInspection
class SinhalaTokenizer():
    def __init__(self):
        self.isolate_punctuations_with_spaces: Boolean = False
        self.punctuation_marks: List[Text] = [
            '.', ',', '\n', ' ', '¸', '‚',
            '"', '/', '-', '|', '\\', '—', '¦',
            '”', '‘', '\'', '“', '’', '´', '´',
            '!', '@', '#', '$', '%', '^', '&', '*', '+', '-', '£', '?', '˜',
            '(', ')', '[', ']', '{', '}',
            ':', ';',
            '\u2013'  # EN - DASH
        ]
        self.invalid_chars: List[Text] = [
            'Ê',
            '\u00a0', '\u2003',  # spaces
            '\ufffd', '\uf020', '\uf073', '\uf06c', '\uf190',  # unknown or invalid unicode chars
            '\u202a', '\u202c', '\u200f'  # direction control chars(for arabic, starting from right etc)
        ]
        self.line_tokenizing_chars: List[Text] = [
            '.', '?', '!', ':', ';', '\u2022'
        ]
        self.punctuations_without_line_tokenizing_chars: List[Text] = [
            ',', '¸', '‚',
            '"', '/', '-', '|', '\\', '—', '¦',
            '”', '‘', '\'', '“', '’', '´', '´',
            '!', '@', '#', '$', '%', '^', '&',
            '*', '+', '-', '£', '?', '˜',
            '(', ')', '[', ']', '{', '}',
            ':', ';',
            '\u2013'
        ]
        self.short_forms: List[Text] = [
            'ඒ.', 'බී.', 'සී.', 'ඩී.', 'ඊ.', 'එෆ්.', 'ජී.', 'එච්.',
            'අයි.', 'ජේ.', 'කේ.', 'එල්.', 'එම්.', 'එන්.', 'ඕ.',
            'පී.', 'කිව්.', 'ආර්.', 'එස්.', 'ටී.', 'යූ.', 'වී.', 'ඩබ.', 'ඩබ්ලිව්.', 'ඩබ්.',
            'එක්ස්.', 'වයි.', 'ඉසෙඩ්.',
            'පෙ.', 'ව.',
            'රු.',
            'පා.',  # parliment
            '0.', '1.', '2.', '3.', '4.', '5.', '6.', '7.', '8.', '9.'
        ]

        # Do not use `short_form_identifier` at `punctuation_marks`
        self.short_form_identifier: Text = '\u0D80'

        #  init ignoring chars
        self.ignoring_chars: List[Text] = [
            '\u200c', '\u0160', '\u00ad', '\u0088', '\uf086', '\u200b', '\ufeff', 'Á', 'À', '®', '¡', 'ª', 'º', '¤',
            '¼', '¾', 'Ó', 'ø', '½', 'ˆ', '', '¢', 'ÿ', '·', 'í', 'Ω', '°', '×', 'µ', '', '~', 'ƒ', '', 'ë', 'Î',
            '‰', '»', '«', 'à', '«', '·', '¨', '…', '⋆', '›', '¥', '⋆', '', '˝', '', '', '◊', 'Ł', '', 'ê', 'Õ', 'Ä',
            'á', 'Ñ', 'Í', '', 'Ñ', 'ç', 'Æ', 'ô', 'Ž', '€', '§', 'Æ', '÷', 'é', '¯', 'é', 'æ', 'î', 'ï', 'ä', 'Ô', 'õ',
            'È', 'Ý', 'ß', 'õ', '', 'ù', 'å', 'Ø', 'Œ', 'Ô', 'Ü', '', 'Ö', 'Û', 'Ï', 'ñ', 'ý', 'œ', '¹', '', 'É', '¯',
            'Ò',
        ]

        # init word tokenizer
        self.word_tokenizer_delims: Text = '[{}]'.format(
            re.escape(''.join(self.punctuation_marks + self.invalid_chars)))

        # init line tokenizer
        self.line_tokenizer_delims: Text = '[{}]'.format(re.escape(''.join(self.line_tokenizing_chars)))

    def tokenize(self, sentence: Text) -> List[Text]:
        # remove ignoring chars from document
        for ignoring_char in self.ignoring_chars:
            if ignoring_char in sentence:
                sentence = sentence.replace(ignoring_char, '')

        # prevent short forms being splitted into separate tokens
        # Eg: පෙ.ව.
        for short_form in self.short_forms:
            representation = short_form[0:-1] + self.short_form_identifier
            sentence = sentence.replace(short_form, representation)

        parts = re.split(r'({})'.format(self.word_tokenizer_delims), sentence)
        tokens = [token.replace(self.short_form_identifier, '.') for token in parts if len(token.strip()) != 0]
        return tokens

    def split_sentences(self, doc: Text, return_sinhala_only: Boolean = False) -> List[Text]:
        # remove ignoring chars from document
        for ignoring_char in self.ignoring_chars:
            if ignoring_char in doc:
                doc = doc.replace(ignoring_char, '')

        # stop words being present with a punctuation at start or end of the word
        # Eg: word?     word,
        if self.isolate_punctuations_with_spaces:  # default is set to FALSE
            for punctuation in self.punctuations_without_line_tokenizing_chars:
                doc = doc.replace(punctuation, ' ' + punctuation + ' ')

        # prevent short forms being splitted into sentences
        # Eg: පෙ.ව.
        for short_form in self.short_forms:
            representation = short_form[0:len(short_form) - 1] + self.short_form_identifier
            doc = doc.replace(short_form, representation)

        sentences = []
        # split lines
        parts = re.split(r'{}'.format(self.line_tokenizer_delims), doc)
        for sentence in parts:
            sentence = sentence.replace(self.short_form_identifier, '.')
            sentence = sentence.strip()
            if contains_sinhala(sentence):  # filter empty sentences and non-sinhala sentences
                sentences.append(sentence)
            elif not return_sinhala_only and len(sentence) != 0:
                sentences.append(sentence)
        return sentences







# -*- coding: utf-8 -*-
from transformers import LongformerTokenizer, LongformerModel
import torch
from sinling import SinhalaTokenizer
from googletrans import Translator
import numpy as np
import string
from sklearn.metrics.pairwise import cosine_similarity

punctuation = set(string.punctuation)
stopwords = ["සහ",
    "සමග",
    "සමඟ",
    "අහා",
    "ආහ්",
    "ආ",
    "ඕහෝ",
    "අනේ",
    "අඳෝ",
    "අපොයි",
    "අපෝ",
    "අයියෝ",
    "ආයි",
    "ඌයි",
    "චී",
    "චිහ්",
    "චික්",
    "හෝ‍",
    "දෝ",
    "දෝහෝ",
    "මෙන්",
    "සේ",
    "වැනි",
    "බඳු",
    "වන්",
    "අයුරු",
    "අයුරින්",
    "ලෙස",
    "වැඩි",
    "ශ්‍රී",
    "හා",
    "ය",
    "නිසා",
    "නිසාවෙන්",
    "බවට",
    "බව",
    "බවෙන්",
    "නම්",
    "වැඩි",
    "සිට",
    "දී",
    "මහා",
    "මහ",
    "පමණ",
    "පමණින්",
    "පමන",
    "වන",
    "විට",
    "විටින්",
    "මේ",
    "මෙලෙස",
    "මෙයින්",
    "ඇති",
    "ලෙස",
    "සිදු",
    "වශයෙන්",
    "යන",
    "සඳහා",
    "මගින්",
    "හෝ‍",
    "ඉතා",
    "ඒ",
    "එම",
    "ද",
    "අතර",
    "විසින්",
    "සමග",
    "පිළිබඳව",
    "පිළිබඳ",
    "තුළ",
    "බව",
    "වැනි",
    "මහ",
    "මෙම",
    "මෙහි",
    "මේ",
    "වෙත",
    "වෙතින්",
    "වෙතට",
    "වෙනුවෙන්",
    "වෙනුවට",
    "වෙන",
    "ගැන",
    "නෑ",
    "අනුව",
    "නව",
    "පිළිබඳ",
    "විශේෂ",
    "දැනට",
    "එහෙන්",
    "මෙහෙන්",
    "එහේ",
    "මෙහේ",
    "ම",
    "තවත්",
    "තව",
    "සහ",
    "දක්වා",
    "ට",
    "ගේ",
    "එ",
    "ක",
    "ක්",
    "බවත්",
    "බවද",
    "මත",
    "ඇතුලු",
    "ඇතුළු",
    "මෙසේ",
    "වඩා",
    "වඩාත්ම",
    "නිති",
    "නිතිත්",
    "නිතොර",
    "නිතර",
    "ඉක්බිති",
    "දැන්",
    "යලි",
    "පුන",
    "ඉතින්",
    "සිට",
    "සිටන්",
    "පටන්",
    "තෙක්",
    "දක්වා",
    "සා",
    "තාක්",
    "තුවක්",
    "පවා",
    "ද",
    "හෝ‍",
    "වත්",
    "විනා",
    "හැර",
    "මිස",
    "මුත්",
    "කිම",
    "කිම්",
    "ඇයි",
    "මන්ද",
    "හෙවත්",
    "නොහොත්",
    "පතා",
    "පාසා",
    "ගානෙ",
    "තව",
    "ඉතා",
    "බොහෝ",
    "වහා",
    "සෙද",
    "සැනින්",
    "හනික",
    "එම්බා",
    "එම්බල",
    "බොල",
    "නම්",
    "වනාහි",
    "කලී",
    "ඉඳුරා",
    "අන්න",
    "ඔන්න",
    "මෙන්න",
    "උදෙසා",
    "පිණිස",
    "සඳහා",
    "අරබයා",
    "නිසා",
    "එනිසා",
    "එබැවින්",
    "බැවින්",
    "හෙයින්",
    "සේක්",
    "සේක",
    "ගැන",
    "අනුව",
    "පරිදි",
    "විට",
    "තෙක්",
    "මෙතෙක්",
    "මේතාක්",
    "තුරු",
    "තුරා",
    "තුරාවට",
    "තුලින්",
    "නමුත්",
    "එනමුත්",
    "වස්",
    "මෙන්",
    "ලෙස",
    "පරිදි",
    "එහෙත්"]  # Define the Sinhala stop word list

class LongformerTextSummarizer:
    def __init__(self):
        self.tokenizer = SinhalaTokenizer()
        self.stopwords = stopwords
        self.punctuation = punctuation
        self.longformer_tokenizer = LongformerTokenizer.from_pretrained('allenai/longformer-base-4096')
        self.longformer_model = LongformerModel.from_pretrained('allenai/longformer-base-4096')
        self.translator = Translator()

    def __preprocess_sentences(self, text_corpus):
        sentences = self.tokenizer.split_sentences(text_corpus)
        sentences = [sent.strip() for sent in sentences]
        cleaned = [sent for sent in sentences if sent not in self.stopwords and sent not in self.punctuation]
        return cleaned, sentences

    def __create_longformer_embeddings(self, cleaned_sentences):
        sentence_embeddings = []
        for sent in cleaned_sentences:
            inputs = self.longformer_tokenizer(sent, return_tensors="pt", max_length=4096, truncation=True, padding='max_length')
            with torch.no_grad():
                output = self.longformer_model(**inputs)
            sentence_embeddings.append(output.last_hidden_state.mean(dim=1).squeeze().numpy())

        return np.array(sentence_embeddings)

    def __append_fullstop(self, text):
        if text and text[-1] not in punctuation:
            return text + '.'
        return text

    def __get_summary(self, top_n_sentences, sentences):
        sentence_organizer = {k: v for v, k in enumerate(sentences)}
        mapped_top_n_sentences = [(cleaned, sentence_organizer[cleaned]) for cleaned in top_n_sentences]
        mapped_top_n_sentences = sorted(mapped_top_n_sentences, key=lambda x: x[1])
        ordered_scored_sentences = [element[0] for element in mapped_top_n_sentences]
        summary = " ".join([self.__append_fullstop(sentence) for sentence in ordered_scored_sentences])
        return summary

    def summarize_by_percent(self, text, percent=20):

        """
        Summarize a given text based on a specified percentage of its length.

        This method leverages the embeddings generated using Longformer to rank the sentences in the text by importance. It then selects the top-ranked sentences based on the specified percentage of the text's total sentences.

        Parameters:
        -----------
        text : str
            The text to be summarized.
        percent : int, optional (default = 20)
            The percentage of the original text's sentences to be retained in the summary. It determines the number of most important sentences to be included.

        Returns:
        --------
        str
            The summarized text.

        Notes:
        ------
        - The method prioritizes retaining the semantic meaning of the original text.
        - Ensure that the text is preprocessed for optimal summarization results.
        - The returned summary will have sentences ordered as they appear in the original text.
        """

        cleaned, original = self.__preprocess_sentences(text)
        sentence_embeddings = self.__create_longformer_embeddings(cleaned)
        average_embedding = np.mean(sentence_embeddings, axis=0)
        cosine_similarities = cosine_similarity(sentence_embeddings, average_embedding.reshape(1, -1)).squeeze()
        total_sentences = len(cleaned)
        top_n = int(total_sentences * (percent / 100))
        top_n_sentences = [cleaned[i] for i in cosine_similarities.argsort()[-top_n:][::-1]]
        return self.__get_summary(top_n_sentences, original)

    def summarize_by_word_count(self, text, word_count=100):

        """
        Summarize a given text based on a specified word count limit.

        This method leverages the embeddings generated using Longformer to rank the sentences in the text by importance. It then selects the top-ranked sentences until the desired word count limit is reached or exceeded.

        Parameters:
        -----------
        text : str
            The text to be summarized.
        word_count : int, optional (default = 100)
            The maximum number of words desired in the summary. The method will select the most important sentences until this word limit is reached or exceeded.

        Returns:
        --------
        str
            The summarized text.

        Notes:
        ------
        - The method prioritizes retaining the semantic meaning of the original text. As a result, the returned summary might slightly exceed the specified word count limit.
        - Ensure that the text is preprocessed for optimal summarization results.
        - The returned summary will have sentences ordered as they appear in the original text.
        """

        cleaned, original = self.__preprocess_sentences(text)
        sentence_embeddings = self.__create_longformer_embeddings(cleaned)
        average_embedding = np.mean(sentence_embeddings, axis=0)
        cosine_similarities = cosine_similarity(sentence_embeddings, average_embedding.reshape(1, -1)).squeeze()

        current_count = 0
        selected_sentences = []
        sorted_sentences = [cleaned[i] for i in cosine_similarities.argsort()[::-1]]
        for sentence in sorted_sentences:
            tokens = self.tokenizer.tokenize(sentence)
            if current_count + len(tokens) <= word_count:
                current_count += len(tokens)
                selected_sentences.append(sentence)
            if current_count >= word_count:
                break

        return self.__get_summary(selected_sentences, original)

    def translate_to_english(self, text):
        """
        Translate the given text from Sinhala (si) to English (en).

        This method leverages a translation service to convert the provided Sinhala text into English. In case of any translation errors, it prints the error message and returns an empty string.

        Parameters:
        -----------
        text : str
            The Sinhala text to be translated to English.

        Returns:
        --------
        str
            The translated English text. If an error occurs during the translation, an empty string is returned.

        Notes:
        ------
        - The quality and accuracy of the translation might vary based on the translation service used.
        - Ensure good internet connectivity for optimal results, as this function might rely on online translation services.
        """
        try:
            return self.translator.translate(text, src='si', dest='en').text
        except Exception as e:
            print(f"Error during translation: {str(e)}")
            return ""




# -*- coding: utf-8 -*-
from transformers import BertTokenizer, BertModel
import torch
from sinling import SinhalaTokenizer
from googletrans import Translator
import numpy as np
import string


punctuation = set(string.punctuation)
stopwords = ["සහ",
             "සමග",
             "සමඟ",
             "අහා",
             "ආහ්",
             "ආ",
             "ඕහෝ",
             "අනේ",
             "අඳෝ",
             "අපොයි",
             "අපෝ",
             "අයියෝ",
             "ආයි",
             "ඌයි",
             "චී",
             "චිහ්",
             "චික්",
             "හෝ‍",
             "දෝ",
             "දෝහෝ",
             "මෙන්",
             "සේ",
             "වැනි",
             "බඳු",
             "වන්",
             "අයුරු",
             "අයුරින්",
             "ලෙස",
             "වැඩි",
             "ශ්‍රී",
             "හා",
             "ය",
             "නිසා",
             "නිසාවෙන්",
             "බවට",
             "බව",
             "බවෙන්",
             "නම්",
             "වැඩි",
             "සිට",
             "දී",
             "මහා",
             "මහ",
             "පමණ",
             "පමණින්",
             "පමන",
             "වන",
             "විට",
             "විටින්",
             "මේ",
             "මෙලෙස",
             "මෙයින්",
             "ඇති",
             "ලෙස",
             "සිදු",
             "වශයෙන්",
             "යන",
             "සඳහා",
             "මගින්",
             "හෝ‍",
             "ඉතා",
             "ඒ",
             "එම",
             "ද",
             "අතර",
             "විසින්",
             "සමග",
             "පිළිබඳව",
             "පිළිබඳ",
             "තුළ",
             "බව",
             "වැනි",
             "මහ",
             "මෙම",
             "මෙහි",
             "මේ",
             "වෙත",
             "වෙතින්",
             "වෙතට",
             "වෙනුවෙන්",
             "වෙනුවට",
             "වෙන",
             "ගැන",
             "නෑ",
             "අනුව",
             "නව",
             "පිළිබඳ",
             "විශේෂ",
             "දැනට",
             "එහෙන්",
             "මෙහෙන්",
             "එහේ",
             "මෙහේ",
             "ම",
             "තවත්",
             "තව",
             "සහ",
             "දක්වා",
             "ට",
             "ගේ",
             "එ",
             "ක",
             "ක්",
             "බවත්",
             "බවද",
             "මත",
             "ඇතුලු",
             "ඇතුළු",
             "මෙසේ",
             "වඩා",
             "වඩාත්ම",
             "නිති",
             "නිතිත්",
             "නිතොර",
             "නිතර",
             "ඉක්බිති",
             "දැන්",
             "යලි",
             "පුන",
             "ඉතින්",
             "සිට",
             "සිටන්",
             "පටන්",
             "තෙක්",
             "දක්වා",
             "සා",
             "තාක්",
             "තුවක්",
             "පවා",
             "ද",
             "හෝ‍",
             "වත්",
             "විනා",
             "හැර",
             "මිස",
             "මුත්",
             "කිම",
             "කිම්",
             "ඇයි",
             "මන්ද",
             "හෙවත්",
             "නොහොත්",
             "පතා",
             "පාසා",
             "ගානෙ",
             "තව",
             "ඉතා",
             "බොහෝ",
             "වහා",
             "සෙද",
             "සැනින්",
             "හනික",
             "එම්බා",
             "එම්බල",
             "බොල",
             "නම්",
             "වනාහි",
             "කලී",
             "ඉඳුරා",
             "අන්න",
             "ඔන්න",
             "මෙන්න",
             "උදෙසා",
             "පිණිස",
             "සඳහා",
             "අරබයා",
             "නිසා",
             "එනිසා",
             "එබැවින්",
             "බැවින්",
             "හෙයින්",
             "සේක්",
             "සේක",
             "ගැන",
             "අනුව",
             "පරිදි",
             "විට",
             "තෙක්",
             "මෙතෙක්",
             "මේතාක්",
             "තුරු",
             "තුරා",
             "තුරාවට",
             "තුලින්",
             "නමුත්",
             "එනමුත්",
             "වස්",
             "මෙන්",
             "ලෙස",
             "පරිදි",
             "එහෙත්"]  # define the sinhala stop word list


class BertTextSummarizer:
    def __init__(self):
        self.tokenizer = SinhalaTokenizer()
        self.stopwords = stopwords
        self.punctuation = punctuation
        self.bert_tokenizer = BertTokenizer.from_pretrained('bert-base-multilingual-uncased')
        self.bert_model = BertModel.from_pretrained('bert-base-multilingual-uncased')

    def __preprocess_sentences(self, text_corpus):
        try:
            sentences = self.tokenizer.split_sentences(text_corpus)
            sentences = [sent.strip() for sent in sentences]
            cleaned = [sent for sent in sentences if sent not in self.stopwords and sent not in self.punctuation]
            return cleaned, sentences
        except Exception as e:
            print(f"Error during preprocessing: {str(e)}")
            return [], []

    def __create_bert_embeddings(self, cleaned_sentences):
        sentence_embeddings = []
        for sent in cleaned_sentences:
            inputs = self.bert_tokenizer(sent, return_tensors="pt", max_length=512, truncation=True,
                                         padding='max_length')
            with torch.no_grad():
                output = self.bert_model(**inputs)
            sentence_embeddings.append(output.last_hidden_state.mean(dim=1).squeeze().numpy())

        if not sentence_embeddings:
            return np.array([]).reshape(0, self.bert_model.config.hidden_size)

        return np.array(sentence_embeddings)

    @staticmethod
    def __append_fullstop(text):
        if text and text[-1] not in punctuation:
            return text + '.'
        return text

    def __get_summary(self, top_n_sentences, sentences):
        try:
            sentence_organizer = {k: v for v, k in enumerate(sentences)}
            mapped_top_n_sentences = [(cleaned, sentence_organizer[cleaned]) for cleaned in top_n_sentences]
            mapped_top_n_sentences = sorted(mapped_top_n_sentences, key=lambda x: x[1])
            ordered_scored_sentences = [element[0] for element in mapped_top_n_sentences]
            summary = " ".join([self.__append_fullstop(sentence) for sentence in ordered_scored_sentences])
            return summary
        except Exception as e:
            print(f"Error during summary extraction: {str(e)}")
            return ""

    def summarize_by_percent(self, text_corpus, percent):
        """
        Summarize a given text based on a specified percentage.

        This method takes a text corpus and summarizes it by selecting the most important sentences based on the embeddings generated using BERT,
        until the desired percentage of the original text's length is reached.

        Parameters:
        -----------
        text_corpus : str
            The text to be summarized.
        percent : float
            The percentage of the most important sentences to be included in the summary. Values should be between 0 and 100.

        Returns:
        --------
        str
            The summarized text.

        Examples:
        ---------
        >>> from sinlingua.summarizer.bert import BertTextSummarizer
        >>> text_summarizer = BertTextSummarizer()
        >>> sample_text = "..."
        >>> summary = text_summarizer.summarize_by_percent(sample_text, percent=50)
        >>> print(summary)

        Notes:
        ------
        - Ensure that the text_corpus is preprocessed for best summarization results.
        - The returned summary will have sentences ordered as they appear in the original text.
        - Ensure the percentage value is between 0 and 100 for accurate results.
        """
        if not (0 <= percent <= 100):
            raise ValueError("Percent value should be between 0 and 100")
        cleaned_sentences, sentences = self.__preprocess_sentences(text_corpus)
        sentence_embeddings = self.__create_bert_embeddings(cleaned_sentences)

        if sentence_embeddings.size == 0:
            return ""

        sentence_scores = np.linalg.norm(sentence_embeddings, axis=1)
        N = int(len(cleaned_sentences) * percent / 100)
        top_n_sentences = [cleaned_sentences[ind] for ind in np.argsort(sentence_scores)[-N:]]
        return self.__get_summary(top_n_sentences, sentences)

    def summarize_by_word_count(self, text_corpus, word_count):
        """
        Summarize a given text  based on a specified word count limit.
        This method leverages the embeddings generated using BERT to rank the sentences in the text corpus by importance. It then selects the top-ranked sentences until the desired word count limit is reached.

        Parameters:
        -----------
        text_corpus : str
            The text to be summarized.
        word_count : int
            The maximum number of words desired in the summary. The method will select the most important sentences until this word limit is reached or exceeded.

        Returns:
        --------
        str
            The summarized text.

        Examples:
        ---------
        >>> from sinlingua.summarizer.bert import BertTextSummarizer
        >>> text_summarizer = BertTextSummarizer()
        >>> sample_text = "අද අප ඔබට කියන්නේ ටිකක් වෙනස් ආරක කතාවකි. මෙය සම්බන්ධ වන්නේ සෞඛ්‍ය ක්‍ෂේත්‍රයට ය. 2018 දෙසැම්බර් මාසයේදී චීනයේ වූහාන් නගරයේ වැසියන් අතර අමුතු ස්වසන රෝගයක් පැතිරෙමින් පවත්නා බව ලී වෙන්ලියෑං නමැති චීන වෛද්‍යවරයෙක් හෙළි කළේය. මේ රෝගය ඊට කලකට පෙර පැතිර ගිය සාර්ස් රෝගය හා සමාන බව එම දොස්තරගේ මතය විය. ඔහු ඒ ගැන සිය මිතුරන්ට දැන්වූ අතර ඒ පිළිබඳ ආරංචිය චීන බලධාරීන්ගේ කනට ගියේ ය. චීන ආණ්ඩුව ඔහු අත්අඩංගුවට ගත් අතර බොරු ආරංචි පතුරුවා චීනයේ කලබල ඇති කිරීමට තැත් කරන බව කියමින් හිර කූඩුවේද දැම්මේ ය. මෙයින් දින කීපයකට පසු කොවිඩ් 19 වැළඳී දොස්තර ලී මිය ගියේ ය. එබැවින් ඔහු බොරු කියා චීනය තුළ කැළඹීමක් ඇති කිරීමට තැත් නොකළ බව අලුතින්ම කීමට චීන ආණ්ඩුවට සිදුවිය."
        >>>
        >>> summary = text_summarizer.summarize_by_word_count(sample_text, word_count=30)
        >>> print(summary)

        Notes:
        ------
        - The method prioritizes retaining the semantic meaning of the original text. As a result, the returned summary might slightly exceed the specified word count limit.
        - Ensure that the text_corpus is preprocessed for optimal summarization results.
        - The returned summary will have sentences ordered as they appear in the original text.
        """

        cleaned_sentences, sentences = self.__preprocess_sentences(text_corpus)
        sentence_embeddings = self.__create_bert_embeddings(cleaned_sentences)

        if sentence_embeddings.size == 0:
            return ""

        sentence_scores = np.linalg.norm(sentence_embeddings, axis=1)
        sorted_sentences = [cleaned_sentences[ind] for ind in np.argsort(sentence_scores)[::-1]]

        N = 0
        total_words = 0
        for sent in sorted_sentences:
            total_words += len(sent.split())
            if total_words <= word_count:
                N += 1
            else:
                break
        top_n_sentences = sorted_sentences[:N]
        return self.__get_summary(top_n_sentences, sentences)

    @staticmethod
    def translate_to_english(text):
        """
        Translate a given Sinhala text (typically a summary) into English.

        This method uses the Google Translate API (through googletrans) to translate the given Sinhala text into English. It's typically used to translate the output of the `summarize_by_percent` or `summarize_by_word_count` methods.

        Parameters:
        -----------
        text : str
            The Sinhala text to be translated, often a summary produced by the other methods in this class.

        Returns:
        --------
        str
            The translated text in English.

        Examples:
        ---------
        >>> from sinlingua.summarizer.bert import BertTextSummarizer
        >>> text_summarizer = BertTextSummarizer()
        >>> sample_text = "අද අප ඔබට කියන්නේ ටිකක් වෙනස් ආරක කතාවකි. මෙය සම්බන්ධ වන්නේ සෞඛ්‍ය ක්‍ෂේත්‍රයට ය. 2018 දෙසැම්බර් මාසයේදී චීනයේ වූහාන් නගරයේ වැසියන් අතර අමුතු ස්වසන රෝගයක් පැතිරෙමින් පවත්නා බව ලී වෙන්ලියෑං නමැති චීන වෛද්‍යවරයෙක් හෙළි කළේය. මේ රෝගය ඊට කලකට පෙර පැතිර ගිය සාර්ස් රෝගය හා සමාන බව එම දොස්තරගේ මතය විය. ඔහු ඒ ගැන සිය මිතුරන්ට දැන්වූ අතර ඒ පිළිබඳ ආරංචිය චීන බලධාරීන්ගේ කනට ගියේ ය. චීන ආණ්ඩුව ඔහු අත්අඩංගුවට ගත් අතර බොරු ආරංචි පතුරුවා චීනයේ කලබල ඇති කිරීමට තැත් කරන බව කියමින් හිර කූඩුවේද දැම්මේ ය. මෙයින් දින කීපයකට පසු කොවිඩ් 19 වැළඳී දොස්තර ලී මිය ගියේ ය. එබැවින් ඔහු බොරු කියා චීනය තුළ කැළඹීමක් ඇති කිරීමට තැත් නොකළ බව අලුතින්ම කීමට චීන ආණ්ඩුවට සිදුවිය."
        >>>
        >>> summarized_text = text_summarizer.summarize_by_percent(sample_text, percent=20)
        >>> translated_text = text_summarizer.translate_to_english(summarized_text)
        >>> print(translated_text)

        Notes:
        ------
        - This method utilizes the unofficial Google Translate API (googletrans). Ensure you have a stable internet connection for best results.
        - For production-level applications, consider using the official Google Cloud Translation API.
        """
        try:
            translator = Translator()
            translation = translator.translate(text, src='si', dest='en')
            return translation.text
        except Exception as e:
            print(f"Error during translation: {str(e)}")
            return text





from sinlingua.grammar_rule.grammar_rules import GrammarRules
from sinlingua.grammar_rule.rule_based_1 import FirstPerson
from sinlingua.grammar_rule.rule_based_2 import SecondPersonSingular
from sinlingua.grammar_rule.rule_based_3 import SecondPersonPlural
from sinlingua.grammar_rule.rule_based_4 import FourthPerson
from sinlingua.grammar_rule.rule_based_future_1 import FirstPersonFuture
from sinlingua.grammar_rule.rule_based_plural import PluralSubject
from sinlingua.grammar_rule.rule_based_plural_past import PluralSubjectPast
from sinlingua.grammar_rule.rule_based_singular import SingularSubject
from sinlingua.grammar_rule.rule_based_past_1 import PastFirstPerson
from sinlingua.grammar_rule.rule_based_past_2 import PastSecondPersonSingular
from sinlingua.grammar_rule.rule_based_past_3 import PastSecondPersonPlural
from sinlingua.grammar_rule.mask import PredictNoun


class GrammarMain:
    def __init__(self):
        pass

    @staticmethod
    def mapper(sentence: str):
        first_person = FirstPerson()
        output1 = first_person.common_function(sentence)
        # print(output1)
        # call the grammar rules for first person

        second_person_singular = SecondPersonSingular()
        output2 = second_person_singular.common_function(sentence)
        # print(output2)
        # call the grammar rules for second person singular

        second_person_plural = SecondPersonPlural()
        output3 = second_person_plural.common_function(sentence)
        # print(output3)
        # call the grammar rules for second person plural

        first_person_future = FirstPersonFuture()
        output4 = first_person_future.common_function(sentence)
        # print(output4)
        # call the grammar rules for first person future

        plural_subject = PluralSubject()
        output5 = plural_subject.common_function(sentence)
        # print(output5)
        # call the grammar rules for plural subjects

        singular_subject = SingularSubject()
        output6 = singular_subject.common_function(sentence)
        # print(output6)
        # call the grammar rules for Singular subjects

        past_first_person = PastFirstPerson()
        output7 = past_first_person.common_function(sentence)
        # print(output7)
        # call the grammar rules for first person past

        past_second_person_singular = PastSecondPersonSingular()
        output8 = past_second_person_singular.common_function(sentence)
        # print(output8)
        # call the grammar rules for second person past singular

        past_second_person_plural = PastSecondPersonPlural()
        output9 = past_second_person_plural.common_function(sentence)
        # print(output9)
        # call the grammar rules for second person past plural

        noun_predict = PredictNoun()
        output10 = noun_predict.common_function(sentence)
        # print(output10)
        # call the grammar rules for predict noun

        plural_subject_past = PluralSubjectPast()
        output11 = plural_subject_past.common_function(sentence)
        # print(output11)
        # call the grammar rules for past plural

        fourth_person = FourthPerson()
        output12 = fourth_person.common_function(sentence)
        # print(output12)
        # call the grammar rules for fourth person

        grammar_rule = GrammarRules()
        # create parent class object
        validated1 = grammar_rule.output(output1[0])
        validated2 = grammar_rule.output(output2[0])
        validated3 = grammar_rule.output(output3[0])
        validated4 = grammar_rule.output(output4[0])
        validated5 = grammar_rule.output(output5[0])
        validated6 = grammar_rule.output(output6[0])
        validated7 = grammar_rule.output(output7[0])
        validated8 = grammar_rule.output(output8[0])
        validated9 = grammar_rule.output(output9[0])
        validated11 = grammar_rule.output(output11[0])
        validated12 = grammar_rule.output(output12[0])
        # validated10 = grammar_rule.output(output10)
        # call function for every grammar rule at once to get correct output

        # if validated1 is None and validated2 is None and validated3 is None and validated4 is None and validated5 is None and validated6 is None and validated7 is None and validated8 is None and validated9 is None:
        #     pass
        if validated1 is not None and validated7 is not None:
            if output1[1] < output7[1]:
                return output7[0]
            if output1[1] > output7[1]:
                return output1[0]
            if output1[1] == output7[1]:
                return output7[0]
        elif validated1 is not None:
            return output1[0]
        elif validated4 is not None:
            return output4[0]
        elif validated7 is not None:
            return output7[0]
        elif validated2 is not None and validated8 is not None:
            if output2[1] < output8[1]:
                return output8[0]
            if output2[1] > output8[1]:
                return output2[0]
            if output2[1] == output8[1]:
                return output2[0]
        elif validated8 is not None:
            return output8[0]
        elif validated2 is not None:
            return output2[0]
        elif validated3 is not None and validated9 is not None:
            if output3[1] < output9[1]:
                return output9[0]
            if output3[1] > output9[1]:
                return output3[0]
            if output3[1] == output9[1]:
                return output3[0]
        elif validated9 is not None:
            return output9[0]
        elif validated3 is not None:
            return output3[0]
        elif validated12 is not None:
            return output12[0]
        elif validated5 is not None and validated6 is not None and validated11 is not None:
            if output5[1] < output6[1]:
                if output6[1] < output11[1]:
                    return output11[0]
                return output6[0]
            if output5[1] > output6[1]:
                if output5[1] < output11[1]:
                    return output11[0]
                return output5[0]
            if output5[1] == output6[1]:
                return output6[0]
        elif validated5 is not None and validated6 is not None:
            if output5[1] < output6[1]:
                return output6[0]
            if output5[1] > output6[1]:
                return output5[0]
        elif validated5 is not None:
            return output5[0]
        elif validated6 is not None:
            return output6[0]
        elif validated11 is not None:
            return output11[0]
        # if function returns any correct output is displayed


# if __name__ == "__main__":
    # obj = GrammarMain()
    # sent = "මා සතුන්ට දානයක් දෙන්න හිතන් ඉන්නවා"
    # out = obj.mapper(sentence=sent)
    # print(out)

# මා සතුන්ට දානයක් දෙන්න හිතන් ඉන්නවා
# මා සතුන්ට දානයක් දෙන්න හිතන් ඉන්වා
# අපි පොසොන් පෝයට දන්සලක් සංවිධානය කරනවා
# අපි පොසොන් පෝයට දන්සලක් සංවිධානය කරවා
# අප පොසොන් පෝයට දන්සලක් සංවිධානය කරවා
# ඇය පිටිසර පාසලක උත්සහයෙන් උගන්වනවා
# ඇය පිටිසර පාසලක උත්සහයෙන් උගන්නවා
# ඈ පිටිසර පාසලක උත්සහයෙන් උගන්නවා
# ඔහු ඉතා ආදරයෙන් දෙමාපියන් නමදිනව
# ඔවුන් නොයෙක් ආකාරයේ ඇඳුම් මහනවාලු
# ඔවුන් නොයෙක් ආකාරයේ ඇඳුම් මහනව
# ඔවුහු නොයෙක් ආකාරයේ ඇඳුම් මහනව
# අප හෙට මේ වෙලාවට නිදාගෙන හිඳීවි
# මම මේ ටික හොඳට බලන් කියලා දෙන්නම්
# ගොවියෝ මහන්සි වී රටට සහල් සපයනවා
# සංගමයේ සාමාජිකයින් සමිතිය අවසන් බැවින් විසිය යනව
# ඇය මල් නෙලුවා
# ඔහු මල් නෙලුව



import pandas as pd
from typing import Tuple


class ManualTransliterator:
    def __init__(self):
        pass

    @staticmethod
    def generate_coordinates(text: str, max_columns: int = 20) -> pd.DataFrame:
        try:
            paragraphs = text.split("\n\n")  # Split text into paragraphs
            num_paragraphs = len(paragraphs)

            # Create an empty DataFrame
            df = pd.DataFrame()

            for p_idx, paragraph in enumerate(paragraphs):
                words = paragraph.split()
                num_words = len(words)
                num_rows = (num_words + max_columns - 1) // max_columns  # Calculate number of rows needed

                paragraph_df = pd.DataFrame(index=range(num_rows), columns=range(max_columns))

                row = 0
                col = 0
                for idx, word in enumerate(words):
                    # Handle full stop
                    if word.endswith('.'):
                        paragraph_df.at[row, col] = word[:-1]  # Remove the full stop and store in the current cell
                        col += 1  # Move to the next column
                        paragraph_df.at[row, col] = '.'  # Place the full stop in the next cell
                        col += 1  # Move to the next column
                    else:
                        paragraph_df.at[row, col] = word
                        col += 1  # Move to the next column

                    # Check if the maximum column limit is reached
                    if col >= max_columns:
                        row += 1
                        col = 0

                # Reset index of paragraph_df
                paragraph_df.reset_index(drop=True, inplace=True)

                # Concatenate with df
                df = pd.concat([df, paragraph_df], axis=0)

                # Add an empty row between paragraphs
                if p_idx < num_paragraphs - 1:
                    empty_row = pd.Series([None] * max_columns)
                    df = pd.concat([df, pd.DataFrame([empty_row])], axis=0)

                # Reset index of df
                df.reset_index(drop=True, inplace=True)

            return df

        except Exception as e:
            print(f"An error occurred: {str(e)}")
            return pd.DataFrame()  # Return an empty DataFrame in case of error

    @staticmethod
    def replace_cells(dataframe: pd.DataFrame, replacement_dict: dict) -> Tuple[pd.DataFrame, list]:
        try:
            changes = []  # Nested list to store changes [(row, col), original, changed]

            for key, value in replacement_dict.items():
                row, col = key

                if pd.notna(dataframe.iat[row, col]):
                    original = dataframe.iat[row, col]
                    dataframe.iat[row, col] = value
                    changes.append([(row, col), original, value])

            return dataframe, changes

        except Exception as e:
            print(f"An error occurred in replace_cells: {str(e)}")
            return dataframe, []  # Return the original DataFrame and an empty changes list

    @staticmethod
    def undo_changes(dataframe: pd.DataFrame, changes: list) -> pd.DataFrame:
        try:
            for change in changes:
                (row, col), original, _ = change
                dataframe.iat[row, col] = original

            return dataframe

        except Exception as e:
            print(f"An error occurred in undo_changes: {str(e)}")
            return dataframe  # Return the original DataFrame in case of error

    @staticmethod
    def reconstruct_text(dataframe: pd.DataFrame) -> str:
        try:
            paragraphs = []
            paragraph = []

            # Iterate over each row in the DataFrame
            for _, row in dataframe.iterrows():
                sentence = []
                for col in row:
                    if pd.notna(col):
                        if col == ".":
                            sentence.append(".")
                        else:
                            sentence.append(col)

                if len(sentence) > 0:
                    paragraph.extend(sentence)
                else:
                    if len(paragraph) > 0:
                        paragraphs.append(" ".join(paragraph))
                        paragraph = []

            if len(paragraph) > 0:
                paragraphs.append(" ".join(paragraph))

            # Join paragraphs with double line breaks and replace spaces around full stops
            constructed_text = "\n\n".join(paragraphs)
            constructed_text = constructed_text.replace(" .", ".").replace("\n\n", "\n")
            return constructed_text

        except Exception as e:
            print(f"An error occurred in reconstruct_text: {str(e)}")
            return ""  # Return an empty string in case of error

    @staticmethod
    def to_csv(dataframe: pd.DataFrame, file: str = "dataframe.csv") -> None:
        try:
            # Save the DataFrame to a CSV file
            dataframe.to_csv(file, index=True)
        except Exception as e:
            print(f"An error occurred in to_csv: {str(e)}")

    @staticmethod
    def manual_mask(dataframe: pd.DataFrame, coordinates: list) -> Tuple[str, list]:
        try:
            changes = []  # To store changes [(row, col), original]
            for coord in coordinates:
                row, col = coord
                original_text = dataframe.iat[row, col]  # Get the original text
                changes.append([(row, col), original_text])

                # Mask the specified cell with "<mask>"
                dataframe.iat[row, col] = "<mask>"

            # Use the changes list to undo the masking after reconstruction
            reconstructed_text = ManualTransliterator.reconstruct_text(dataframe=dataframe)
            return reconstructed_text, changes

        except Exception as e:
            print(f"An error occurred in manual_masking: {str(e)}")
            return "", []  # Return an empty string and an empty list in case of error


from sinlingua.grammar_rule.grammar_rules import GrammarRules, translate_sinhala_to_english
from googletrans import Translator
from sinlingua.src.grammar_rule_resources import verbs, nouns_subject_plural


class PluralSubject(GrammarRules):
    def common_function(self, sentence):
        grammar_obj = GrammarRules()
        global conjugated_sentence
        prefixes = ["මා", "අපි", "මම", "ම", "අප", "ඔහු", "ඇය", "ඈ", "ඔවුන්", "ඔවුහු"]
        conjugated_verb = ''
        returned_string_verb = grammar_obj.find_similar_words(verbs, sentence)
        newsentence = sentence
        verb_checked = returned_string_verb[0]
        actual_word = returned_string_verb[1]
        returned_string_subject = grammar_obj.find_similar_words(nouns_subject_plural, sentence)
        subject_checked = returned_string_subject[0]
        actual_subject = returned_string_subject[1]
        ratio = returned_string_subject[2]
        if returned_string_verb[0]:
            # Extract the verb stem
            verb_stem = verb_checked[:-3]
            if returned_string_subject[0]:
                sentence = sentence.replace(actual_subject, subject_checked)
            words = sentence.split()
            if returned_string_subject[0]:
                if verb_stem[-1] in ["්"]:
                    verb_stem = verb_stem[:-1]
                    verb_stem = verb_stem + "ි"
                # Adding "ති" for the stem
                conjugated_verb = verb_stem + "ති"
                # Remove the last word (verb) from the sentence
                words.remove(actual_word)
                words.append(conjugated_verb)
                # Reconstruct the sentence
                conjugated_sentence = " ".join(words)
            elif any(sentence.startswith(prefix) or sentence.split()[1] == prefix for prefix in prefixes):
                conjugated_verb = ''

            else:
                words.remove(actual_word)
                conjugated_verb = verb_stem + "මින් ඇත"
                words.append(conjugated_verb)
                # Reconstruct the sentence
                conjugated_sentence = " ".join(words)
                # print("Sorry.........This may be wrong. No enough data to process_plural")
                """"# Translate Sinhala sentence to English
                english_translation = translate_sinhala_to_english(' '.join(words))
                print(english_translation)"""

        if conjugated_verb == '':
            return "Try Again..Incomplete sentence. No enough data to process", ratio

        else:
            return conjugated_sentence, ratio



import os
import numpy as np
from sklearn.svm import SVC
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.model_selection import train_test_split
import pickle
from sinlingua.config import RESOURCE_PATH

# Load the dataset
data_file = 'IT20167264/word_set/singular_plural_train_data.text'
data = np.loadtxt(os.path.join(RESOURCE_PATH, data_file), delimiter=',', dtype=str, encoding='utf-8')

# Split the dataset into input features and labels
X = data[:, 0]  # Sinhala nouns
y = data[:, 1]  # Singular or Plural labels

# Convert labels to numeric values: Singular -> 0, Plural -> 1
y = np.where(y == 'Singular', 0, 1)

# Split the data into training and testing sets
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.11, random_state=42)

# Feature extraction
vectorizer = CountVectorizer()
X_train_features = vectorizer.fit_transform(X_train)
X_test_features = vectorizer.transform(X_test)

# Train the SVM model
svm_model = SVC(kernel='linear')
svm_model.fit(X_train_features, y_train)

# Evaluate the model
accuracy = svm_model.score(X_test_features, y_test)
print(f"Accuracy: {accuracy}")

# Make predictions on new data
# new_data = ["අය", "හෙසිනි"]
# new_data_features = vectorizer.transform(new_data)
# predictions = svm_model.predict(new_data_features)

# Print the predictions
# for noun, prediction in zip(new_data, predictions):
# label = "Singular" if prediction == 0 else "Plural"
# print(f"Noun: {noun} - Prediction: {label}")

# Serialize and save the object as a pickle file
dict = {"model": svm_model, "vectorizer": vectorizer}
with open('../../resources/pickels/svm_singular_plural.pkl', 'wb') as file:
    pickle.dump(dict, file)

print("Python object converted to a pickle file.")






from fuzzywuzzy import fuzz
from googletrans import Translator


class GrammarRules:
    @staticmethod
    def find_similar_words(list_items, input_string):
        words = input_string.split()
        max_similarity = 0
        similar_word = None
        actual_word_of_string = None
        for line in list_items:
            line = line.strip()  # Remove leading/trailing whitespace
            for word in words:
                similarity_ratio = fuzz.ratio(word, line)
                if similarity_ratio >= 75 and similarity_ratio > max_similarity:
                    max_similarity = similarity_ratio
                    similar_word = line
                    actual_word_of_string = word
        return similar_word, actual_word_of_string, max_similarity

    def common_function(self, sentence):
        # Common implementation of the function
        pass

    @staticmethod
    def output(output):
        if output != "Try Again..Incomplete sentence. No enough data to process":
            return output


def translate_sinhala_to_english(sentence):
    translator = Translator()
    translated = translator.translate(sentence, src='si', dest='en')
    return translated.text



import os
from typing import Tuple
import openai
import json
import time
import requests
from sinlingua.src.grammar_rule_resources import grammar_rule_llm_config


class LLMConfig:
    def __init__(self, api_key: str = None, org_key: str = None):
        self.json_data = grammar_rule_llm_config
        if api_key is not None and org_key is not None:
            self.json_data["api_key"] = api_key
            self.json_data["org_key"] = org_key

    # @staticmethod
    # def __read_json_config(file_path: str) -> dict:
    #     try:
    #         # Read JSON configuration file and return the data as dictionary
    #         with open(os.path.join(RESOURCE_PATH, file_path), 'r', encoding='utf-8') as json_file:
    #             json_data_c = json.load(json_file)
    #         return json_data_c
    #     except Exception as e:
    #         # Handle exceptions while reading JSON configuration
    #         print(f"Error while reading JSON configuration file '{file_path}': {str(e)}")
    #         return {}

    def __get_llm_response(self, text: str, level: int) -> str:
        completion = None
        try:
            # Set up API key and organization for OpenAI
            openai.api_key = self.json_data["api_key"]
            openai.organization = self.json_data["org_key"]

            # Create user prompt using provided text and level
            user_prompt = self.json_data["Prompts"][level]["content"].replace("{{word}}", text)

            # Check if the provided text is empty
            if not text.strip():
                raise ValueError("Text is empty. Please provide a valid text string.")

            success = False
            while not success:
                try:
                    # Create a ChatCompletion request to GPT-3
                    completion = openai.ChatCompletion.create(
                        model=self.json_data["model"],
                        messages=[
                            {
                                "role": self.json_data["Prompts"][level]['role'],
                                "content": user_prompt
                            }
                        ],
                        n=1,
                        temperature=self.json_data['temperature'],
                        max_tokens=self.json_data['max_tokens'],
                        top_p=self.json_data['Top_P'],
                        frequency_penalty=self.json_data['Frequency_penalty'],
                        presence_penalty=self.json_data['Presence_penalty']
                    )
                    success = True
                except Exception as e:
                    # Handle exceptions during GPT-3 request
                    sleep_time = 2
                    time.sleep(sleep_time)
                    print("Error:", e)
                    print("Retrying...")

            result = completion.choices[0].message.content
            sleep_time = 2
            time.sleep(sleep_time)  # To avoid rate limit
            return result
        except Exception as e:
            # Handle exceptions during GPT response processing
            print(f"Error in GPT response for text '{text}': {str(e)}")
            return ""

    def llm_check(self, word: str, level: int) -> dict:
        response = self.__get_llm_response(text=word, level=level)
        dictionary = json.loads(response)
        return dictionary

    @staticmethod
    def berto_predict(sentence: str) -> Tuple[str, str]:
        API_TOKEN = "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"
        API_URL = "https://api-inference.huggingface.co/models/keshan/SinhalaBERTo"
        headers = {"Authorization": f"Bearer {API_TOKEN}"}

        def query(payload):
            response = requests.post(API_URL, headers=headers, json=payload)
            return response.json()

        output = query({
            "inputs": sentence,
        })

        lines = output
        line = lines[0]
        return line["sequence"], line["token_str"]

    @staticmethod
    def berto_predict_top(sentence: str) -> list:
        API_TOKEN = "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"
        API_URL = "https://api-inference.huggingface.co/models/keshan/SinhalaBERTo"
        headers = {"Authorization": f"Bearer {API_TOKEN}"}

        def query(payload):
            response = requests.post(API_URL, headers=headers, json=payload)
            return response.json()

        output = query({
            "inputs": sentence,
        })

        list_out = []
        lines = output
        for line in lines:
            list_out.append([line["sequence"], line["token_str"]])

        return list_out



from fuzzywuzzy import fuzz
from sinlingua.grammar_rule.grammar_rules import GrammarRules
from sinlingua.src.grammar_rule_resources import verbs


class SecondPersonPlural(GrammarRules):
    def common_function(self, sentence):
        global conjugated_sentence
        grammar_obj = GrammarRules()
        conjugated_verb = ''
        returned_string_verb = grammar_obj.find_similar_words(verbs, sentence)
        verb_checked = returned_string_verb[0]
        actual_word = returned_string_verb[1]
        ratio = returned_string_verb[2]
        if returned_string_verb[0]:
            if verb_checked == 'ගන්නවා':
                verb_checked = 'ගනිනවා'
            elif verb_checked == 'පෙන්නනනවා':
                verb_checked = 'පෙන්වනවා'
            elif verb_checked == 'ගන්නනනවා':
                verb_checked = 'ගන්වනවා'
            elif verb_checked == 'නිදාගන්නවා':
                verb_checked = 'නිදාගනිනවා'
            # Extract the verb stem
            verb_stem = verb_checked[:-3]
            # Remove the last word (verb) from the sentence
            words = sentence.split()
            words.remove(actual_word)
            # Check if the sentence starts with "ඔවුන්" or "ඔවුහු"
            if sentence.split()[0] in ["ඔවුන්", "ඔවුහු"] or (
                    len(sentence.split()) > 1 and sentence.split()[1] in ["ඔවුන්", "ඔවුහු"]):
                if len(words) > 1 and words[1] in ["ඔවුන්", "ඔවුන්"]:
                    words.pop(0)
                conjugated_verb = verb_stem + "ති"
                # Append the conjugated verb to the sentence
            words.append(conjugated_verb)
            # Reconstruct the sentence
            conjugated_sentence = " ".join(words)
        if conjugated_verb == '':
            return "Try Again..Incomplete sentence. No enough data to process", ratio

        else:
            return conjugated_sentence, ratio



from sinlingua.src.preprocessor_resources import stop_words
import os


class StopWordRemover:
    def __init__(self):
        self.stop_words = stop_words

    def remove_stop_words(self, text):
        words = text.split()
        remaining_words = [word for word in words if word not in self.stop_words]
        remaining_text = ' '.join(remaining_words)
        return remaining_text






import os
import openai
import json
import time
from sinlingua.singlish.rulebased_transliterator import RuleBasedTransliterator
from sinlingua.src.singlish_resources import config_data


class HybridTransliterator:
    def __init__(self, api_key: str = None, org_key: str = None, prompt_masking: str = None, prompt_suggestion: str = None):
        # config_file = "config.json"
        # self.json_data = self.__read_json_config(file_path=config_file)
        self.json_data = config_data
        if api_key is not None and org_key is not None:
            self.json_data["api_key"] = api_key
            self.json_data["org_key"] = org_key
        if prompt_masking is not None:
            self.json_data["Prompts"][0]["content"] = prompt_masking
        if prompt_suggestion is not None:
            self.json_data["Prompts"][1]["content"] = prompt_suggestion

    def view_prompt(self, level: int):
        print(self.json_data["Prompts"][level]["content"])

    # @staticmethod
    # def __read_json_config(file_path: str) -> dict:
    #     try:
    #         # Read JSON configuration file and return the data as dictionary
    #         with open(os.path.join(RESOURCE_PATH, file_path), 'r', encoding='utf-8') as json_file:
    #             json_data_c = json.load(json_file)
    #         return json_data_c
    #     except Exception as e:
    #         # Handle exceptions while reading JSON configuration
    #         print(f"Error while reading JSON configuration file '{file_path}': {str(e)}")
    #         return {}

    def __get_gpt_response(self, text: str, level: int, word: str = "") -> str:
        completion = None
        try:
            # Set up API key and organization for OpenAI
            openai.api_key = self.json_data["api_key"]
            openai.organization = self.json_data["org_key"]

            # Create user prompt using provided text and level
            user_prompt = self.json_data["Prompts"][level]["content"].replace("{{masked-sentence}}", text).replace(
                "{{misspelled-word}}", word)

            # Check if the provided text is empty
            if not text.strip():
                raise ValueError("Text is empty. Please provide a valid text string.")

            success = False
            while not success:
                try:
                    # Create a ChatCompletion request to GPT-3
                    completion = openai.ChatCompletion.create(
                        model=self.json_data["model"],
                        messages=[
                            {
                                "role": self.json_data["Prompts"][level]['role'],
                                "content": user_prompt
                            }
                        ],
                        n=1,
                        temperature=self.json_data['temperature'],
                        max_tokens=self.json_data['max_tokens'],
                        top_p=self.json_data['Top_P'],
                        frequency_penalty=self.json_data['Frequency_penalty'],
                        presence_penalty=self.json_data['Presence_penalty']
                    )
                    success = True
                except Exception as e:
                    # Handle exceptions during GPT-3 request
                    sleep_time = 2
                    time.sleep(sleep_time)
                    print("Error:", e)
                    print("Retrying...")

            result = completion.choices[0].message.content
            sleep_time = 2
            time.sleep(sleep_time)  # To avoid rate limit
            return result
        except Exception as e:
            # Handle exceptions during GPT response processing
            print(f"Error in GPT response for text '{text}': {str(e)}")
            return ""

    @staticmethod
    def __remove_duplicates(input_list: list) -> list:
        unique_words = set()
        result = []

        for word in input_list:
            # Remove duplicates from the input list
            if word not in unique_words:
                unique_words.add(word)
                result.append(word)

        return result

    def transliterator(self, text: str) -> str:
        try:
            # Call parent transliterator to get the initial output
            rule_based = RuleBasedTransliterator()
            output = rule_based.transliterator(text=text)

            # Process output sentence by sentence
            paragraphs = output.split('\n')
            merged_para = []

            for paragraph in paragraphs:
                sentences = paragraph.split(".")
                merged_sentence = []

                for sentence in sentences:
                    try:
                        if sentence == "":
                            continue
                        # Get GPT response for level 0
                        gpt_response_1 = self.__get_gpt_response(text=sentence, level=0)
                        dictionary_1 = json.loads(gpt_response_1)

                        primary_list_item = 0  # Initialize the index for the while loop
                        dictionary_list = dictionary_1["word_list"]
                        dictionary_list = self.__remove_duplicates(dictionary_list)

                        while primary_list_item < len(dictionary_1["word_list"]):
                            for list_item in dictionary_list:

                                # Check for multiple occurrences of the same word
                                if sentence.count(list_item) > 1:
                                    sentence = sentence.replace(list_item, "<mask>", 1)
                                    dictionary_1["word_list"].append(list_item)
                                    dictionary_list.remove(list_item)
                                    dictionary_list.append(list_item)

                                    # Get GPT response for level 1
                                    gpt_response_2 = self.__get_gpt_response(text=sentence, level=1, word=list_item)
                                    dictionary_2 = json.loads(gpt_response_2)

                                    if list_item in dictionary_2:
                                        sentence = sentence.replace("<mask>", dictionary_2[list_item])
                                    else:
                                        sentence = sentence.replace("<mask>", list_item)
                                    break

                                # Check for single occurrence of the word
                                elif sentence.count(list_item) == 1:
                                    sentence = sentence.replace(list_item, "<mask>", 1)
                                    gpt_response_2 = self.__get_gpt_response(text=sentence, level=1, word=list_item)
                                    dictionary_2 = json.loads(gpt_response_2)

                                    if list_item in dictionary_2:
                                        sentence = sentence.replace("<mask>", dictionary_2[list_item])
                                    else:
                                        sentence = sentence.replace("<mask>", list_item)
                                    break

                            primary_list_item += 1

                        merged_sentence.append(sentence)
                    except Exception as e:
                        print(f"Error processing sentence: {str(e)}")
                para = ". ".join(merged_sentence)
                merged_para.append(para)
            full_text = "\n".join(merged_para)
            return full_text

        except Exception as e:
            print(f"Error in transliterator: {str(e)}")
            return text

    def machine_mask(self, text: str,) -> str:
        try:
            # Call parent transliterator to get the initial output
            rule_based = RuleBasedTransliterator()
            output = rule_based.transliterator(text=text)

            # Process output sentence by sentence
            paragraphs = output.split('\n')
            merged_para = []

            for paragraph in paragraphs:
                sentences = paragraph.split(".")
                merged_sentence = []

                for sentence in sentences:
                    try:
                        if sentence == "":
                            continue
                        # Get GPT response for level 0
                        gpt_response_1 = self.__get_gpt_response(text=sentence, level=0)
                        dictionary_1 = json.loads(gpt_response_1)

                        primary_list_item = 0  # Initialize the index for the while loop
                        dictionary_list = dictionary_1["word_list"]
                        dictionary_list = self.__remove_duplicates(dictionary_list)

                        while primary_list_item < len(dictionary_1["word_list"]):
                            for list_item in dictionary_list:

                                # Check for multiple occurrences of the same word
                                if sentence.count(list_item) > 1:
                                    sentence = sentence.replace(list_item, "<mask>", 1)
                                    dictionary_1["word_list"].append(list_item)
                                    dictionary_list.remove(list_item)
                                    dictionary_list.append(list_item)
                                    break

                                # Check for single occurrence of the word
                                elif sentence.count(list_item) == 1:
                                    sentence = sentence.replace(list_item, "<mask>", 1)
                                    break

                            primary_list_item += 1

                        merged_sentence.append(sentence)
                    except Exception as e:
                        print(f"Error processing sentence: {str(e)}")
                para = ". ".join(merged_sentence)
                merged_para.append(para)
            full_text = "\n".join(merged_para)
            return full_text
        except Exception as e:
            print(f"An error occurred in machine_mask: {str(e)}")
            return ""  # Return an empty string and an empty list in case of error

    def machine_suggest(self, text: str, changes: list) -> str:
        try:
            # Process output sentence by sentence
            paragraphs = text.split('\n')
            merged_para = []

            flag = 0
            for paragraph in paragraphs:
                sentences = paragraph.split(".")
                merged_sentence = []

                for sentence in sentences:
                    try:
                        if sentence == "":
                            continue
                        mask_count_in_sentence = sentence.count("<mask>")
                        sentence = sentence
                        mask_word = ""
                        for n in range(0, mask_count_in_sentence):
                            for m in range(n, mask_count_in_sentence):
                                ind = flag + m
                                if n == m:
                                    sentence = sentence.replace("<mask>", "####", 1)
                                    mask_word = changes[ind][1]
                                else:
                                    sentence = sentence.replace("<mask>", changes[ind][1], 1)

                            sentence = sentence.replace("####", "<mask>")
                            gpt_response = self.__get_gpt_response(text=sentence, level=1, word=mask_word)
                            dictionary = json.loads(gpt_response)
                            if mask_word in dictionary:
                                sentence = sentence.replace("<mask>", dictionary[mask_word])
                            else:
                                sentence = sentence.replace("<mask>", mask_word)

                        flag += mask_count_in_sentence
                        merged_sentence.append(sentence)
                    except Exception as e:
                        print(f"Error processing sentence: {str(e)}")
                para = ". ".join(merged_sentence)
                merged_para.append(para)
            full_text = "\n".join(merged_para)
            return full_text
        except Exception as e:
            print(f"An error occurred in machine_suggest: {str(e)}")
            return ""  # Return an empty string and an empty list in case of error


# -*- coding: utf-8 -*-
from transformers import DistilBertTokenizer, DistilBertModel, LongformerTokenizer, LongformerModel
import torch
from sinling import SinhalaTokenizer
from googletrans import Translator
import numpy as np
import string
from sklearn.metrics.pairwise import cosine_similarity

punctuation = set(string.punctuation)
stopwords = ["සහ",
    "සමග",
    "සමඟ",
    "අහා",
    "ආහ්",
    "ආ",
    "ඕහෝ",
    "අනේ",
    "අඳෝ",
    "අපොයි",
    "අපෝ",
    "අයියෝ",
    "ආයි",
    "ඌයි",
    "චී",
    "චිහ්",
    "චික්",
    "හෝ‍",
    "දෝ",
    "දෝහෝ",
    "මෙන්",
    "සේ",
    "වැනි",
    "බඳු",
    "වන්",
    "අයුරු",
    "අයුරින්",
    "ලෙස",
    "වැඩි",
    "ශ්‍රී",
    "හා",
    "ය",
    "නිසා",
    "නිසාවෙන්",
    "බවට",
    "බව",
    "බවෙන්",
    "නම්",
    "වැඩි",
    "සිට",
    "දී",
    "මහා",
    "මහ",
    "පමණ",
    "පමණින්",
    "පමන",
    "වන",
    "විට",
    "විටින්",
    "මේ",
    "මෙලෙස",
    "මෙයින්",
    "ඇති",
    "ලෙස",
    "සිදු",
    "වශයෙන්",
    "යන",
    "සඳහා",
    "මගින්",
    "හෝ‍",
    "ඉතා",
    "ඒ",
    "එම",
    "ද",
    "අතර",
    "විසින්",
    "සමග",
    "පිළිබඳව",
    "පිළිබඳ",
    "තුළ",
    "බව",
    "වැනි",
    "මහ",
    "මෙම",
    "මෙහි",
    "මේ",
    "වෙත",
    "වෙතින්",
    "වෙතට",
    "වෙනුවෙන්",
    "වෙනුවට",
    "වෙන",
    "ගැන",
    "නෑ",
    "අනුව",
    "නව",
    "පිළිබඳ",
    "විශේෂ",
    "දැනට",
    "එහෙන්",
    "මෙහෙන්",
    "එහේ",
    "මෙහේ",
    "ම",
    "තවත්",
    "තව",
    "සහ",
    "දක්වා",
    "ට",
    "ගේ",
    "එ",
    "ක",
    "ක්",
    "බවත්",
    "බවද",
    "මත",
    "ඇතුලු",
    "ඇතුළු",
    "මෙසේ",
    "වඩා",
    "වඩාත්ම",
    "නිති",
    "නිතිත්",
    "නිතොර",
    "නිතර",
    "ඉක්බිති",
    "දැන්",
    "යලි",
    "පුන",
    "ඉතින්",
    "සිට",
    "සිටන්",
    "පටන්",
    "තෙක්",
    "දක්වා",
    "සා",
    "තාක්",
    "තුවක්",
    "පවා",
    "ද",
    "හෝ‍",
    "වත්",
    "විනා",
    "හැර",
    "මිස",
    "මුත්",
    "කිම",
    "කිම්",
    "ඇයි",
    "මන්ද",
    "හෙවත්",
    "නොහොත්",
    "පතා",
    "පාසා",
    "ගානෙ",
    "තව",
    "ඉතා",
    "බොහෝ",
    "වහා",
    "සෙද",
    "සැනින්",
    "හනික",
    "එම්බා",
    "එම්බල",
    "බොල",
    "නම්",
    "වනාහි",
    "කලී",
    "ඉඳුරා",
    "අන්න",
    "ඔන්න",
    "මෙන්න",
    "උදෙසා",
    "පිණිස",
    "සඳහා",
    "අරබයා",
    "නිසා",
    "එනිසා",
    "එබැවින්",
    "බැවින්",
    "හෙයින්",
    "සේක්",
    "සේක",
    "ගැන",
    "අනුව",
    "පරිදි",
    "විට",
    "තෙක්",
    "මෙතෙක්",
    "මේතාක්",
    "තුරු",
    "තුරා",
    "තුරාවට",
    "තුලින්",
    "නමුත්",
    "එනමුත්",
    "වස්",
    "මෙන්",
    "ලෙස",
    "පරිදි",
    "එහෙත්"]  # Define the Sinhala stopword list

class FLongformerTextSummarizer:
    def __init__(self):
        self.tokenizer = SinhalaTokenizer()
        self.stopwords = stopwords
        self.punctuation = punctuation
        self.distilbert_tokenizer = DistilBertTokenizer.from_pretrained('distilbert-base-multilingual-cased')
        self.distilbert_model = DistilBertModel.from_pretrained('distilbert-base-multilingual-cased')
        self.longformer_tokenizer = LongformerTokenizer.from_pretrained('allenai/longformer-base-4096')
        self.longformer_model = LongformerModel.from_pretrained('allenai/longformer-base-4096')
        self.translator = Translator()

    def __preprocess_sentences(self, text_corpus):
        sentences = self.tokenizer.split_sentences(text_corpus)
        sentences = [sent.strip() for sent in sentences]
        cleaned = [sent for sent in sentences if sent not in self.stopwords and sent not in self.punctuation]
        return cleaned, sentences

    def __create_distilbert_embeddings(self, cleaned_sentences):
        sentence_embeddings = []
        for sent in cleaned_sentences:
            inputs = self.distilbert_tokenizer.encode_plus(sent, return_tensors="pt", max_length=512, truncation=True, padding='max_length')
            with torch.no_grad():
                output = self.distilbert_model(**inputs)
            sentence_embeddings.append(output.last_hidden_state.mean(dim=1).squeeze().numpy())
        return np.array(sentence_embeddings)

    def __create_longformer_embeddings(self, cleaned_sentences):
        sentence_embeddings = []
        for sent in cleaned_sentences:
            inputs = self.longformer_tokenizer.encode_plus(sent, return_tensors="pt", max_length=4096, truncation=True, padding='max_length')
            with torch.no_grad():
                output = self.longformer_model(**inputs)
            sentence_embeddings.append(output.last_hidden_state.mean(dim=1).squeeze().numpy())
        return np.array(sentence_embeddings)

    def __append_fullstop(self, text):
        if text and text[-1] not in punctuation:
            return text + '.'
        return text

    def __get_summary(self, embeddings, original_sentences, word_limit):
        avg_embedding = np.mean(embeddings, axis=0).reshape(1, -1)
        cosine_similarities = cosine_similarity(embeddings, avg_embedding).squeeze()
        current_count = 0
        selected_sentences = []
        sorted_sentences = [original_sentences[i] for i in cosine_similarities.argsort()[::-1]]
        for sentence in sorted_sentences:
            tokens = self.tokenizer.tokenize(sentence)
            if current_count + len(tokens) <= word_limit:
                current_count += len(tokens)
                selected_sentences.append(sentence)
            if current_count >= word_limit:
                break
        sentence_organizer = {k: v for v, k in enumerate(original_sentences)}
        mapped_sentences = [(cleaned, sentence_organizer[cleaned]) for cleaned in selected_sentences]
        mapped_sentences = sorted(mapped_sentences, key=lambda x: x[1])
        ordered_sentences = [element[0] for element in mapped_sentences]
        summary = " ".join([self.__append_fullstop(sentence) for sentence in ordered_sentences])
        return summary

    def refined_summarize_by_word_count(self, text, distilbert_word_limit=300, longformer_word_limit=100):
        """
        Generate a refined summary of the given text using DistilBert followed by Longformer embeddings.

        The method first leverages DistilBert embeddings to create an initial summary and then uses Longformer embeddings to refine the summary further. This two-step approach aims to retain the core semantic meaning of the original text while providing a concise summary.

        Parameters:
        -----------
        text : str
            The text to be summarized.
        distilbert_word_limit : int, optional (default = 300)
            The word limit for the summary generated using DistilBert embeddings.
        longformer_word_limit : int, optional (default = 100)
            The word limit for the final refined summary generated using Longformer embeddings.

        Returns:
        --------
        str
            The refined summarized text.

        Notes:
        ------
        - The method prioritizes retaining the semantic meaning of the original text.
        - Ensure that the text is preprocessed for optimal summarization results.
        - The returned summary will have sentences ordered as they appear in the original text.
        """
        cleaned, original = self.__preprocess_sentences(text)
        distilbert_embeddings = self.__create_distilbert_embeddings(cleaned)
        initial_summary = self.__get_summary(distilbert_embeddings, original, distilbert_word_limit)
        cleaned_refined, _ = self.__preprocess_sentences(initial_summary)
        longformer_embeddings = self.__create_longformer_embeddings(cleaned_refined)
        final_summary = self.__get_summary(longformer_embeddings, cleaned_refined, longformer_word_limit)
        return final_summary

    def translate_to_english(self, text):
        """
        Translate the provided text from Sinhala to English.

        Utilizes the Translator class to perform the translation. It might fail for various reasons such as exceeding translation API limits, network issues, etc.

        Parameters:
        -----------
        text : str
            The text in Sinhala to be translated to English.

        Returns:
        --------
        str
            The translated text in English.

        Notes:
        ------
        - Ensure that you have an active internet connection for the translation process.
        - Translation quality might vary based on the complexity of the text.
        - Handle potential exceptions that might arise from the translation API.
        """
        try:
            return self.translator.translate(text, src='si', dest='en').text
        except Exception as e:
            print(f"Error during translation: {str(e)}")
            return ""




import requests
from sinlingua.grammar_rule.grammar_rules import GrammarRules
from sinlingua.grammar_rule.LLM_config import LLMConfig
from sinlingua.src.grammar_rule_resources import verbs


class PredictNoun(GrammarRules):
    def common_function(self, sentence):
        # llm = LLMConfig()

        grammar_obj = GrammarRules()
        global conjugated_sentence
        conjugated_verb = ''
        returned_string_verb = grammar_obj.find_similar_words(verbs, sentence)
        verb_checked = returned_string_verb[0]
        actual_word = returned_string_verb[1]
        ratio = returned_string_verb[2]
        if returned_string_verb[0]:
            # Extract the verb stem
            verb_stem = verb_checked[:-1]
            # Remove the last word (verb) from the sentence
            words = sentence.split()
            words.remove(actual_word)

            # Get gpt output for singular/plural
            # llm_dict = llm.llm_check(word=words[0], level=0)
            # if llm_dict[words[0]]:
            #     sing_plu = llm_dict[words[0]]
            # else:
            #     sing_plu = "singular"

            # print(words[0])
            # print(sing_plu)



from fuzzywuzzy import fuzz
from sinlingua.grammar_rule.grammar_rules import GrammarRules
from sinlingua.src.grammar_rule_resources import past_verbs


class PastSecondPersonSingular(GrammarRules):
    def common_function(self, sentence):
        global conjugated_sentence
        grammar_obj = GrammarRules()

        conjugated_verb = ''
        returned_string_verb = grammar_obj.find_similar_words(past_verbs, sentence)
        # call the function find verb of sentence
        # returned_string_verb = find_similar_words(file_path_for_verb, sentence)
        verb_checked = returned_string_verb[0]
        actual_word = returned_string_verb[1]
        ratio = returned_string_verb[2]
        if returned_string_verb[0]:
            # Extract the verb stem
            # verb_stem = verb_checked[:-2]
            # Remove the last word (verb) from the sentence
            words = sentence.split()
            words.remove(actual_word)
            # Check if the sentence starts with "ඇය" or "ඈ"
            if sentence.split()[0] in ["ඇය", "ඈ"] or (len(sentence.split()) > 1 and sentence.split()[1] in ["ඇය", "ඈ"]):
                if len(words) > 1 and words[1] in ["ඇය", "ඈ"]:
                    words.pop(0)
                # verb_stem = verb_checked[:-2]
                conjugated_verb = verb_checked + "ය"
                # Add the first word as "ඇය" to the sentence
                words[0] = "ඇය"

            # Check if the sentence starts with "ඔහු"
            elif sentence.split()[0] in ["ඔහු"] or (len(sentence.split()) > 1 and sentence.split()[1] in ["ඔහු"]):
                if len(words) > 1 and words[1] in ["ඔහු"]:
                    words.pop(0)
                verb_stem = verb_checked[:-1]
                conjugated_verb = verb_stem + "ේය"
                # Add the first word as "ඔහු" to the sentence
                words[0] = "ඔහු"

            # Append the conjugated verb to the sentence
            words.append(conjugated_verb)
            # Reconstruct the sentence
            conjugated_sentence = " ".join(words)
        if conjugated_verb == '':
            return "Try Again..Incomplete sentence. No enough data to process", ratio

        else:
            return conjugated_sentence, ratio



# -*- coding: utf-8 -*-
from sklearn.feature_extraction.text import TfidfVectorizer
from sinling import SinhalaTokenizer
from googletrans import Translator
import numpy as np
import string


punctuation = set(string.punctuation)
stopwords = [
    "සහ",
    "සමග",
    "සමඟ",
    "අහා",
    "ආහ්",
    "ආ",
    "ඕහෝ",
    "අනේ",
    "අඳෝ",
    "අපොයි",
    "අපෝ",
    "අයියෝ",
    "ආයි",
    "ඌයි",
    "චී",
    "චිහ්",
    "චික්",
    "හෝ‍",
    "දෝ",
    "දෝහෝ",
    "මෙන්",
    "සේ",
    "වැනි",
    "බඳු",
    "වන්",
    "අයුරු",
    "අයුරින්",
    "ලෙස",
    "වැඩි",
    "ශ්‍රී",
    "හා",
    "ය",
    "නිසා",
    "නිසාවෙන්",
    "බවට",
    "බව",
    "බවෙන්",
    "නම්",
    "වැඩි",
    "සිට",
    "දී",
    "මහා",
    "මහ",
    "පමණ",
    "පමණින්",
    "පමන",
    "වන",
    "විට",
    "විටින්",
    "මේ",
    "මෙලෙස",
    "මෙයින්",
    "ඇති",
    "ලෙස",
    "සිදු",
    "වශයෙන්",
    "යන",
    "සඳහා",
    "මගින්",
    "හෝ‍",
    "ඉතා",
    "ඒ",
    "එම",
    "ද",
    "අතර",
    "විසින්",
    "සමග",
    "පිළිබඳව",
    "පිළිබඳ",
    "තුළ",
    "බව",
    "වැනි",
    "මහ",
    "මෙම",
    "මෙහි",
    "මේ",
    "වෙත",
    "වෙතින්",
    "වෙතට",
    "වෙනුවෙන්",
    "වෙනුවට",
    "වෙන",
    "ගැන",
    "නෑ",
    "අනුව",
    "නව",
    "පිළිබඳ",
    "විශේෂ",
    "දැනට",
    "එහෙන්",
    "මෙහෙන්",
    "එහේ",
    "මෙහේ",
    "ම",
    "තවත්",
    "තව",
    "සහ",
    "දක්වා",
    "ට",
    "ගේ",
    "එ",
    "ක",
    "ක්",
    "බවත්",
    "බවද",
    "මත",
    "ඇතුලු",
    "ඇතුළු",
    "මෙසේ",
    "වඩා",
    "වඩාත්ම",
    "නිති",
    "නිතිත්",
    "නිතොර",
    "නිතර",
    "ඉක්බිති",
    "දැන්",
    "යලි",
    "පුන",
    "ඉතින්",
    "සිට",
    "සිටන්",
    "පටන්",
    "තෙක්",
    "දක්වා",
    "සා",
    "තාක්",
    "තුවක්",
    "පවා",
    "ද",
    "හෝ‍",
    "වත්",
    "විනා",
    "හැර",
    "මිස",
    "මුත්",
    "කිම",
    "කිම්",
    "ඇයි",
    "මන්ද",
    "හෙවත්",
    "නොහොත්",
    "පතා",
    "පාසා",
    "ගානෙ",
    "තව",
    "ඉතා",
    "බොහෝ",
    "වහා",
    "සෙද",
    "සැනින්",
    "හනික",
    "එම්බා",
    "එම්බල",
    "බොල",
    "නම්",
    "වනාහි",
    "කලී",
    "ඉඳුරා",
    "අන්න",
    "ඔන්න",
    "මෙන්න",
    "උදෙසා",
    "පිණිස",
    "සඳහා",
    "අරබයා",
    "නිසා",
    "එනිසා",
    "එබැවින්",
    "බැවින්",
    "හෙයින්",
    "සේක්",
    "සේක",
    "ගැන",
    "අනුව",
    "පරිදි",
    "විට",
    "තෙක්",
    "මෙතෙක්",
    "මේතාක්",
    "තුරු",
    "තුරා",
    "තුරාවට",
    "තුලින්",
    "නමුත්",
    "එනමුත්",
    "වස්",
    "මෙන්",
    "ලෙස",
    "පරිදි",
    "එහෙත්"]  # define the sinhala stop word list


class TFIDFTextSummarizer:
    def __init__(self):
        self.tokenizer = SinhalaTokenizer()
        self.stopwords = stopwords
        self.punctuation = punctuation

    def __preprocess_sentences(self, text):  # stopwords & punctuation handling
        try:
            sentences = self.tokenizer.split_sentences(text)
            sentences = [sent.strip() for sent in sentences]
            cleaned = [sent for sent in sentences if sent not in self.stopwords and sent not in self.punctuation]
            return cleaned, sentences
        except Exception as e:
            print(f"Error during preprocessing: {str(e)}")
            return [], []

    @staticmethod
    def __create_tfidf_model(cleaned_sentences):  # sentences vectorized by the tfidf method
        try:
            tf_idf_vectorizer = TfidfVectorizer(
                min_df=2, max_features=None, strip_accents='unicode', analyzer='word',
                token_pattern=r'\w{1,}', ngram_range=(1, 3), use_idf=1, smooth_idf=1, sublinear_tf=1,
            )
            tf_idf_vectorizer.fit(cleaned_sentences)
            sentence_vectors = tf_idf_vectorizer.transform(cleaned_sentences)
            return tf_idf_vectorizer, sentence_vectors
        except Exception as e:
            print(f"Error during TF-IDF model creation: {str(e)}")
            return None, []

    @staticmethod
    def __append_fullstop(text):   # Add fullstop for end of each sentences
        if text and text[-1] not in punctuation:
            return text + '.'
        return text

    def __get_summary(self, top_n_sentences, sentences):  # ordering sentences & make the summary
        try:
            sentence_organizer = {k: v for v, k in enumerate(sentences)}
            mapped_top_n_sentences = [(cleaned, sentence_organizer[cleaned]) for cleaned in top_n_sentences]
            mapped_top_n_sentences = sorted(mapped_top_n_sentences, key=lambda x: x[1])
            ordered_scored_sentences = [element[0] for element in mapped_top_n_sentences]
            summary = " ".join([self.__append_fullstop(sentence) for sentence in ordered_scored_sentences])
            return summary
        except Exception as e:
            print(f"Error during summary extraction: {str(e)}")
            return ""

    def summarize_by_percent(self, text, percent):
        """
        Summarize a given text corpus based on a specified percentage.

        This method takes a text corpus and summarizes it by selecting the most important sentences until the desired percentage is reached.

        Parameters:
        -----------
        text_corpus : str
            The text to be summarized.
        percent : float
            The percentage of the original text to be retained in the summary.

        Returns:
        --------
        str
            The summarized text.

        Examples:
        ---------
        >>> from sinlingua.summarizer.tf_idf import TFIDFTextSummarizer
        >>> text_summarizer = TFIDFTextSummarizer()
        >>> sample_text = "පාර්ලිමේන්තුවේ කාර්ය මණ්ඩලයේ සිටින ඇතැම් කාන්තාවන් එම කාර්ය මණ්ඩලයේම සිටින පිරිමින් අතින් අපහරණයට ලක්වූ බව අප පසුගිය දිනයේ කියෙව්වෙමු. මේ පිළිබඳ පරීක්‍ෂා කිරීම සඳහා දැනටමත් සාමාජිකයන් තුන්දෙනකුගෙන් සමන්විත කමිටුවක් පත්කොට තිබේ. සමගි ජනබලවේගයේ මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි පෙර කී කාන්තාවන් අපහරණ කළ කාම දුර්ජනයන් විසින් එම චෝදනාව ප්‍රතික්‍ෂේප කරන ලිපියක් අපහරණයට පත්වූ කාන්තාවන් ලවා අත්සන් කර ගැනීමට උත්සාහ කර තිබේ. මෙය ඉතා භයානක තත්ත්වයකි. පාර්ලිමේන්තුවේ සිටින පිරිමි සේවක මණ්ඩලය විශාල නොවන බැවින් පෙර කී ලිංගික පිස්සන් සමූහය කවරේදැයි හඳුනාගැනීමට ඉතා පහසුය. හඳුනාගත් වහාම ඔවුන්ගේ වැඩ තහනම් කළ යුතුය. කථානායකතුමා මේ කාර්යය සම්බන්ධයෙන් දැඩිව සිටින බව අපට විශ්වාසය. පාර්ලිමේන්තු මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි මෙය රට කරවන ප්‍රමුඛ ආයතනය වූ පාර්ලිමේන්තුවට එල්ල කරන ලද මඩ පහරකි. මේ පිළිබඳ තනිව නැගී සිට හඬ නැගීම ගැන රෝහිණී කවිරත්නට ස්තුතිවන්ත වෙමු. මේ අතර පසුගිය දිනවල පාර්ලිමේන්තුවේ මන්ත්‍රීවරියන්ට වාචිකව කරන ලද ලිංගික අඩන්තේට්ටම ගැනද දැනගන්නට ලැබිණ. යුද බිමේ තිබෙන ස්ත්‍රී අපහරණය නමැති නීච ආයුධය පාර්ලිමේන්තුව තුළට ගෙන ඒමට ඉඩදිය යුතු නැත."
        >>>
        >>> summary = text_summarizer.summarize_by_percent(sample_text, percent=20)
        >>> print(summary)
        සමගි ජනබලවේගයේ මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි පෙර කී කාන්තාවන් අපහරණ කළ කාම දුර්ජනයන් විසින් එම චෝදනාව ප්‍රතික්‍ෂේප කරන ලිපියක් අපහරණයට පත්වූ කාන්තාවන් ලවා අත්සන් කර ගැනීමට උත්සාහ කර තිබේ. පාර්ලිමේන්තුවේ සිටින පිරිමි සේවක මණ්ඩලය විශාල නොවන බැවින් පෙර කී ලිංගික පිස්සන් සමූහය කවරේදැයි හඳුනාගැනීමට ඉතා පහසුය.

        Notes:
        ------
        - Ensure that the text_corpus is preprocessed for best summarization results.
        - The returned summary will have sentences ordered as they appear in the original text.
        """
        cleaned_sentences, sentences = self.__preprocess_sentences(text)
        _, sentence_vectors = self.__create_tfidf_model(cleaned_sentences)
        sentence_scores = np.array(sentence_vectors.sum(axis=1)).ravel()
        N = int(len(cleaned_sentences) * percent / 100)
        top_n_sentences = [cleaned_sentences[ind] for ind in np.argsort(sentence_scores, axis=0)[::-1][:N]]
        return self.__get_summary(top_n_sentences, sentences)

    def summarize_by_word_count(self, text, word_count):
        """
            Summarize a given text corpus based on a specified word count.

            This method takes a text corpus and summarizes it by selecting the most important sentences until the desired word count is reached.

            Parameters:
            -----------
            text_corpus : str
                The text to be summarized.
            word_count : int
                The maximum number of words desired in the summary.

            Returns:
            --------
            str
                The summarized text.

            Examples:
            ---------
            >>> from sinlingua.summarizer.tf_idf import TFIDFTextSummarizer
            >>> text_summarizer = TFIDFTextSummarizer()
            >>> sample_text = "පාර්ලිමේන්තුවේ කාර්ය මණ්ඩලයේ සිටින ඇතැම් කාන්තාවන් එම කාර්ය මණ්ඩලයේම සිටින පිරිමින් අතින් අපහරණයට ලක්වූ බව අප පසුගිය දිනයේ කියෙව්වෙමු. මේ පිළිබඳ පරීක්‍ෂා කිරීම සඳහා දැනටමත් සාමාජිකයන් තුන්දෙනකුගෙන් සමන්විත කමිටුවක් පත්කොට තිබේ. සමගි ජනබලවේගයේ මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි පෙර කී කාන්තාවන් අපහරණ කළ කාම දුර්ජනයන් විසින් එම චෝදනාව ප්‍රතික්‍ෂේප කරන ලිපියක් අපහරණයට පත්වූ කාන්තාවන් ලවා අත්සන් කර ගැනීමට උත්සාහ කර තිබේ. මෙය ඉතා භයානක තත්ත්වයකි. පාර්ලිමේන්තුවේ සිටින පිරිමි සේවක මණ්ඩලය විශාල නොවන බැවින් පෙර කී ලිංගික පිස්සන් සමූහය කවරේදැයි හඳුනාගැනීමට ඉතා පහසුය. හඳුනාගත් වහාම ඔවුන්ගේ වැඩ තහනම් කළ යුතුය. කථානායකතුමා මේ කාර්යය සම්බන්ධයෙන් දැඩිව සිටින බව අපට විශ්වාසය. පාර්ලිමේන්තු මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි මෙය රට කරවන ප්‍රමුඛ ආයතනය වූ පාර්ලිමේන්තුවට එල්ල කරන ලද මඩ පහරකි. මේ පිළිබඳ තනිව නැගී සිට හඬ නැගීම ගැන රෝහිණී කවිරත්නට ස්තුතිවන්ත වෙමු. මේ අතර පසුගිය දිනවල පාර්ලිමේන්තුවේ මන්ත්‍රීවරියන්ට වාචිකව කරන ලද ලිංගික අඩන්තේට්ටම ගැනද දැනගන්නට ලැබිණ. යුද බිමේ තිබෙන ස්ත්‍රී අපහරණය නමැති නීච ආයුධය පාර්ලිමේන්තුව තුළට ගෙන ඒමට ඉඩදිය යුතු නැත."
            >>>
            >>> summary = text_summarizer.summarize_by_word_count(sample_text, word_count=500)
            >>> print(summary)
            පාර්ලිමේන්තුවේ කාර්ය මණ්ඩලයේ සිටින ඇතැම් කාන්තාවන් එම කාර්ය මණ්ඩලයේම සිටින පිරිමින් අතින් අපහරණයට ලක්වූ බව අප පසුගිය දිනයේ කියෙව්වෙමු. මේ පිළිබඳ පරීක්‍ෂා කිරීම සඳහා දැනටමත් සාමාජිකයන් තුන්දෙනකුගෙන් සමන්විත කමිටුවක් පත්කොට තිබේ. සමගි ජනබලවේගයේ මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි පෙර කී කාන්තාවන් අපහරණ කළ කාම දුර්ජනයන් විසින් එම චෝදනාව ප්‍රතික්‍ෂේප කරන ලිපියක් අපහරණයට පත්වූ කාන්තාවන් ලවා අත්සන් කර ගැනීමට උත්සාහ කර තිබේ. මෙය ඉතා භයානක තත්ත්වයකි. පාර්ලිමේන්තුවේ සිටින පිරිමි සේවක මණ්ඩලය විශාල නොවන බැවින් පෙර කී ලිංගික පිස්සන් සමූහය කවරේදැයි හඳුනාගැනීමට ඉතා පහසුය. හඳුනාගත් වහාම ඔවුන්ගේ වැඩ තහනම් කළ යුතුය. කථානායකතුමා මේ කාර්යය සම්බන්ධයෙන් දැඩිව සිටින බව අපට විශ්වාසය. පාර්ලිමේන්තු මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි මෙය රට කරවන ප්‍රමුඛ ආයතනය වූ පාර්ලිමේන්තුවට එල්ල කරන ලද මඩ පහරකි. මේ පිළිබඳ තනිව නැගී සිට හඬ නැගීම ගැන රෝහිණී කවිරත්නට ස්තුතිවන්ත වෙමු. මේ අතර පසුගිය දිනවල පාර්ලිමේන්තුවේ මන්ත්‍රීවරියන්ට වාචිකව කරන ලද ලිංගික අඩන්තේට්ටම ගැනද දැනගන්නට ලැබිණ. යුද බිමේ තිබෙන ස්ත්‍රී අපහරණය නමැති නීච ආයුධය පාර්ලිමේන්තුව තුළට ගෙන ඒමට ඉඩදිය යුතු නැත.

            Notes:
            ------
            - Ensure that the text_corpus is preprocessed for best summarization results.
            - The returned summary will have sentences ordered as they appear in the original text.
            """
        cleaned_sentences, sentences = self.__preprocess_sentences(text)
        _, sentence_vectors = self.__create_tfidf_model(cleaned_sentences)
        sentence_scores = np.array(sentence_vectors.sum(axis=1)).ravel()
        sorted_sentences = [cleaned_sentences[ind] for ind in np.argsort(sentence_scores, axis=0)[::-1]]
        N = 0
        total_words = 0
        for sent in sorted_sentences:
            total_words += len(sent.split())
            if total_words <= word_count:
                N += 1
            else:
                break
        top_n_sentences = [cleaned_sentences[ind] for ind in np.argsort(sentence_scores, axis=0)[::-1][:N]]
        return self.__get_summary(top_n_sentences, sentences)

    @staticmethod
    def translate_to_english(text):
        """
        Translate a given Sinhala text (typically a summary) into English.

        This method uses the Google Translate API (through googletrans) to translate the given Sinhala text into English. It's typically used to translate the output of the `summarize_by_percent` or `summarize_by_word_count` methods.

        Parameters:
        -----------
        text : str
            The Sinhala text to be translated, often a summary produced by the other methods in this class.

        Returns:
        --------
        str
            The translated text in English.

        Examples:
        ---------
        >>> from sinlingua.summarizer.tf_idf import TFIDFTextSummarizer
        >>> text_summarizer = TFIDFTextSummarizer()
        >>> sample_text = "පාර්ලිමේන්තුවේ කාර්ය මණ්ඩලයේ සිටින ඇතැම් කාන්තාවන් එම කාර්ය මණ්ඩලයේම සිටින පිරිමින් අතින් අපහරණයට ලක්වූ බව අප පසුගිය දිනයේ කියෙව්වෙමු. මේ පිළිබඳ පරීක්‍ෂා කිරීම සඳහා දැනටමත් සාමාජිකයන් තුන්දෙනකුගෙන් සමන්විත කමිටුවක් පත්කොට තිබේ. සමගි ජනබලවේගයේ මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි පෙර කී කාන්තාවන් අපහරණ කළ කාම දුර්ජනයන් විසින් එම චෝදනාව ප්‍රතික්‍ෂේප කරන ලිපියක් අපහරණයට පත්වූ කාන්තාවන් ලවා අත්සන් කර ගැනීමට උත්සාහ කර තිබේ. මෙය ඉතා භයානක තත්ත්වයකි. පාර්ලිමේන්තුවේ සිටින පිරිමි සේවක මණ්ඩලය විශාල නොවන බැවින් පෙර කී ලිංගික පිස්සන් සමූහය කවරේදැයි හඳුනාගැනීමට ඉතා පහසුය. හඳුනාගත් වහාම ඔවුන්ගේ වැඩ තහනම් කළ යුතුය. කථානායකතුමා මේ කාර්යය සම්බන්ධයෙන් දැඩිව සිටින බව අපට විශ්වාසය. පාර්ලිමේන්තු මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි මෙය රට කරවන ප්‍රමුඛ ආයතනය වූ පාර්ලිමේන්තුවට එල්ල කරන ලද මඩ පහරකි. මේ පිළිබඳ තනිව නැගී සිට හඬ නැගීම ගැන රෝහිණී කවිරත්නට ස්තුතිවන්ත වෙමු. මේ අතර පසුගිය දිනවල පාර්ලිමේන්තුවේ මන්ත්‍රීවරියන්ට වාචිකව කරන ලද ලිංගික අඩන්තේට්ටම ගැනද දැනගන්නට ලැබිණ. යුද බිමේ තිබෙන ස්ත්‍රී අපහරණය නමැති නීච ආයුධය පාර්ලිමේන්තුව තුළට ගෙන ඒමට ඉඩදිය යුතු නැත."
        >>>
        >>> summarized_text = text_summarizer.summarize_by_percent(sample_text, percent=20)
        >>> translated_text = text_summarizer.translate_to_english(summarized_text)
        >>> print(translated_text)
        සමගි ජනබලවේගයේ මන්ත්‍රී රෝහිණී කවිරත්න කියන පරිදි පෙර කී කාන්තාවන් අපහරණ කළ කාම දුර්ජනයන් විසින් එම චෝදනාව ප්‍රතික්‍ෂේප කරන ලිපියක් අපහරණයට පත්වූ කාන්තාවන් ලවා අත්සන් කර ගැනීමට උත්සාහ කර තිබේ. පාර්ලිමේන්තුවේ සිටින පිරිමි සේවක මණ්ඩලය විශාල නොවන බැවින් පෙර කී ලිංගික පිස්සන් සමූහය කවරේදැයි හඳුනාගැනීමට ඉතා පහසුය.

        Notes:
        ------
        - This method utilizes the unofficial Google Translate API (googletrans). Ensure you have a stable internet connection for best results.
        - For production-level applications, consider using the official Google Cloud Translation API.
        """
        try:
            translator = Translator()
            translation = translator.translate(text, src='si', dest='en')
            return translation.text
        except Exception as e:
            print(f"Error during translation: {str(e)}")
            return text





import os
import pandas as pd
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score, mean_squared_error
from sinlingua.config import RESOURCE_PATH

file_path = 'IT20167264/word_set/singular_plural_train_data.text'
# Load data from text file
with open(os.path.join(RESOURCE_PATH, file_path), 'r', encoding='utf-8') as file:
    lines = file.readlines()

# Assuming the first column contains the nouns and the second column contains the labels
nouns = []
labels = []
for line in lines:
    line = line.strip()
    columns = line.split(',')
    nouns.append(columns[0])
    labels.append(columns[1])

# Convert labels to numerical values (0 for singular, 1 for plural)
labels = [0 if label.lower() == 'singular' else 1 for label in labels]

# Split data into training and testing sets
X_train, X_test, y_train, y_test = train_test_split(nouns, labels, test_size=0.11, random_state=42)

# Create a CountVectorizer to convert text into numerical features
vectorizer = CountVectorizer()
X_train_vec = vectorizer.fit_transform(X_train)
X_test_vec = vectorizer.transform(X_test)

# Create a logistic regression model
model = LogisticRegression()

# Train the model
model.fit(X_train_vec, y_train)

# Predict on the test set
y_pred = model.predict(X_test_vec)

# Calculate accuracy
accuracy = accuracy_score(y_test, y_pred)
print("Accuracy:", accuracy)

# Create a table of predictions and actual values
results = pd.DataFrame({'Actual': y_test, 'Prediction': y_pred})

# Calculate the mean squared error
mse = mean_squared_error(y_test, y_pred)
print("Mean Squared Error:", mse)

# Display the table of predictions and actual values
print(results)
print(y_test)



from setuptools import setup
from pathlib import Path

this_directory = Path(__file__).parent
setup(
    name='sinlingua',
    version='0.1.1',
    packages=['sinlingua', 'sinlingua.src', 'sinlingua.singlish', 'sinlingua.summarizer', 'sinlingua.grammar_rule',
              'sinlingua.preprocessor', 'sinlingua.sinhala_audio'],
    url='https://github.com/SupunGurusinghe/SinlinguaDocumentation/blob/main/README.md',
    license='MIT',
    author='Supun Gurusinghe, Sandaruwini Galappaththi, Supun Sarada Wijesinghe, Binura Yasodya',
    author_email='supunsameeran@gmail.com, sandaruwinigalappaththi@gmail.com, saradawijesinghe@gmail.com, binurayasodya24@gmail.com',
    description='Package for Sinhala data processing',
    python_requires=">=3.7.0",
    long_description=(this_directory / "README.md").read_text(),
    long_description_content_type='text/markdown',
    install_requires=[
        "fuzzywuzzy==0.18.0",
        "gensim==4.3.1",
        "googletrans==4.0.0-rc1",
        "numpy==1.22.4",
        "openai==0.27.8",
        "pandas==2.0.3",
        "pygtrie==2.5.0",
        "Requests==2.31.0",
        "scikit_learn==1.1.3",
        "setuptools==56.0.0",
        "sinling==0.3.6",
        "SpeechRecognition==3.10.0",
        "torch==2.0.1",
        "transformers==4.32.0"
    ]
)



# -*- coding: utf-8 -*-
import os
import functools
from typing import Union
import pygtrie as trie
from sinlingua.config import RESOURCE_PATH
from sinlingua.src.preprocessor_resources import suffix_list_dependent_vowels, suffixes_list, stem_dictionary

dependent_vowels = {
    "අ": "ා",
    "ආ": "ා",
    "ඇ": "ැ",
    "ඈ": "ෑ",
    "ඉ": 'ි',
    "ඊ": 'ී',
    "උ": 'ු',
    "ඌ": 'ූ',
    "එ": 'ෙ',
    "ඒ": 'ේ',
    "ඔ": 'ො',
    "ඕ": 'ෝ'
}


def _load_stem_dictionary():
    stem_list = []
    with open(os.path.join(RESOURCE_PATH, 'stem_dictionary.txt'), 'r', encoding='utf-8') as fp:
        for line in fp.read().split('\n'):
            try:
                base, suffix = line.strip().split('\t')
                stem_list.append([base, suffix])
            except ValueError as _:
                pass
    return stem_list


def _load_suffixes(file: str):
    suffixes = trie.Trie()
    with open(os.path.join(RESOURCE_PATH, file), 'r', encoding='utf-8') as fp:
        for suffix in fp.read().split('\n'):
            suffixes[suffix[::-1]] = suffix
    return suffixes


def _load_lists(file: str) -> list:
    with open(os.path.join(RESOURCE_PATH, file), 'r', encoding='utf-8') as file:
        lines = file.readlines()  # Read all lines into a list
    lines = [line.strip() for line in lines]
    return lines


class SinhalaStemmer:
    def __init__(self):
        super().__init__()
        self.stem_list = stem_dictionary
        # self.suffixes = _load_suffixes(file='suffixes_list.txt')
        self.suffixes_normal = suffixes_list
        self.suffixes_dependent_vowels = suffix_list_dependent_vowels

    def step_one(self, text: str):
        for items in self.stem_list:
            if items[0] == text:
                return items[1]
            else:
                return text

    def step_two(self, text: str) -> str:
        for subtext in self.suffixes_normal:
            if text.endswith(subtext):
                last_index = text.rfind(subtext)
                modified_text = text[:last_index] + text[last_index + len(subtext):]
                return modified_text
        return text

    def step_three(self, text: str) -> str:
        for subtext in self.suffixes_normal:
            before_subtext = subtext[:1]
            subtext = subtext[1:]
            if text.endswith(subtext):
                last_index = text.rfind(subtext)
                modified_text = text[:last_index] + text[last_index + len(subtext):]
                if before_subtext in dependent_vowels:
                    modified_text = modified_text + dependent_vowels[before_subtext]
                    return modified_text
        return text

    def step_four(self, text: str) -> str:
        for subtext in self.suffixes_dependent_vowels:
            if text.endswith(subtext):
                last_index = text.rfind(subtext)
                modified_text = text[:last_index] + text[last_index + len(subtext):]
                return modified_text
        return text

    # def stem(self, word):
    #     if word in self.stem_dictionary:
    #         return self.stem_dictionary[word]
    #     else:
    #         suffix = self.suffixes.longest_prefix(word[::-1]).key
    #         if suffix is not None:
    #             return word[0:-len(suffix)], word[len(word) - len(suffix):]
    #         else:
    #             return word, ''

    @functools.singledispatchmethod
    def stemmer(self, x: Union[str, list]):
        raise NotImplementedError

    @stemmer.register(str)
    def _stemmer_(self, x: str):
        stemmed_list = []
        paragraphs = x.split("\n")
        for paragraph in paragraphs:
            paragraph = paragraph.strip()
            sentences = paragraph.split(".")
            for sentence in sentences:
                sentence = sentence.strip()
                words = sentence.split(" ")
                for word in words:
                    word = word.strip()
                    step1_out = self.step_one(text=word)
                    if step1_out == word:
                        step2_out = self.step_two(text=step1_out)
                        step3_out = self.step_three(text=step2_out)
                        step4_out = self.step_four(text=step3_out)
                        stemmed_list.append(step4_out)
                        continue
                    stemmed_list.append(step1_out)
        return stemmed_list

    @stemmer.register(list)
    def _stemmer_(self, x: list):
        stemmed_list = []
        for word in x:
            word = word.strip()
            step1_out = self.step_one(text=word)
            print(step1_out)
            if step1_out == word:
                step2_out = self.step_two(text=step1_out)
                step3_out = self.step_three(text=step2_out)
                stemmed_list.append(step3_out)
                continue
            stemmed_list.append(step1_out)
        return stemmed_list


if __name__ == "__main__":
    stemmer_obj = SinhalaStemmer()

    input = "විදුලි බිල හා ජල බිල තුන් හතර ගුණයකින් වැඩිවී, වැඩි කලක් යන්නට මත්තෙන් නැවත විදුලි අර්බුදයක හා ජල අර්බුදයක බර ජනතාවගේ හිස මත කඩා වැටී ඇත. විදුලිඅර්බුද මේ ඊයේ පෙරේදා දෙවනවරටත් ජල ගාස්තු සියයට 50 කට වැඩි ගණනකින් වැඩිකර ඇත. එසේම, නුදුරු අනාගතයේම විදුලි කප්පාදුවකට රජය අර අඳින බව පෙනෙන්නට තිබේ. අමාත්‍ය කංචන විජේසේකරගේ කතා බහෙන්ද ඒ බව ඉඟි කෙරේ. මීට හේතුව රටට බලපා ඇති දැඩි නියං තත්ත්වයයි. මේ නිසා ජල සහ විදුලි කප්පාදුවක් ළඟ ළඟම එන බව අප සියල්ලෝම තේරුම් ගත යුතුව"
    output = stemmer_obj.stemmer(input)
    print(output)



from sinlingua.singlish.rulebased_transliterator import RuleBasedTransliterator
from gensim.models import KeyedVectors


class MachineTransliterator(RuleBasedTransliterator):
    def __init__(self, model: str):
        """
        Initialize the MachineTransliterator.

        Parameters:
        -----------
        model: str
            Path to the FastText model file.

        Returns:
        --------
        None

        Example:
        --------
        >>> model_path = "../../resources/models/cc.si.300.vec"
        >>> transliterator = MachineTransliterator(model=model_path)
        """
        super().__init__()
        fasttext_path = model
        self.fasttext_model = KeyedVectors.load_word2vec_format(fasttext_path)

    def __get_best_suggestions(self, word: str) -> list:
        try:
            if word in self.fasttext_model:
                # Get most similar words from the FastText model
                most_similar_words = self.fasttext_model.most_similar(word)
                best_suggestions = [similar_word[0] for similar_word in most_similar_words]
                return best_suggestions
            else:
                return []
        except Exception as e:
            # Handle exceptions and return an empty list
            print(f"An error occurred while getting suggestions: {str(e)}")
            return []

    def transliterator(self, text: str) -> str:
        """
        Transliterate the input text using the rule-based approach and apply suggestions from the FastText model.

        This method takes Singlish text as input, transliterates it into the Sinhala script using the rule-based approach,
        and then applies word suggestions from the FastText model to enhance the transliteration quality.

        The method first uses the rule-based transliteration approach to convert Singlish text into Sinhala script.
        Then, for each word, it retrieves the best suggestions from the FastText model and replaces the original word
        with the top suggestion if available.

        Parameters:
        -----------
        text : str
            The Singlish text to be transliterated.

        Returns:
        --------
        str
            The transliterated text in the Sinhala script with FastText-based suggestions applied.

        Example:
        ---------
        >>> from sinlingua.singlish.rulebased_transliterator import RuleBasedTransliterator
        >>>
        >>> model_path = "../../resources/models/cc.si.300.vec"
        >>> transliterator = MachineTransliterator(model=model_path)
        >>> input_phrase = "oyaata kohomada"
        >>>
        >>> out = transliterator.transliterator(text=input_phrase)
        >>> print(out)
        ඔයාට කොහොමද

        Notes:
        ------
        - The method combines the rule-based and machine learning approaches for better transliteration results.
        - Make sure the FastText model has been initialized and loaded before calling this method.
        """
        try:
            # Call parent transliterator to get the initial output
            output = super().transliterator(text)

            # Process output word by word
            paragraphs = output.split('\n')
            similarity_paragraphs = []
            for paragraph in paragraphs:
                words = paragraph.split()
                similarity_words = []

                for word in words:
                    # Get the best suggestions for each word
                    best_suggestions = self.__get_best_suggestions(word)
                    if best_suggestions:
                        # Replace word with the best suggestion
                        suggested_word = best_suggestions[0]  # Assuming you only need the top suggestion
                        word = suggested_word
                    similarity_words.append(word)

                # Reconstruct the processed text
                processed_text = ' '.join(similarity_words)
                similarity_paragraphs.append(processed_text)

            output_text = '\n'.join(similarity_paragraphs)
            return output_text

        except Exception as e:
            # Handle exceptions and return the original input text
            print(f"An error occurred during transliteration: {str(e)}")
            return text